#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.utils.translation import gettext_lazy as _

from horizon import exceptions

from openstack_dashboard.api import base
from openstack_dashboard.usage import base as usage

from manila_ui.api import manila

#
# Add extra pie charts to project/compute overview
#


class ManilaUsage(usage.ProjectUsage):

    def get_manila_limits(self):
        """Get share limits if manila is enabled."""
        if not base.is_service_enabled(self.request, 'share'):
            return
        try:
            self.limits.update(manila.tenant_absolute_limits(self.request))
        except Exception:
            msg = _("Unable to retrieve share limit information.")
            exceptions.handle(self.request, msg)
        return

    def get_limits(self):
        super(self.__class__, self).get_limits()
        self.get_manila_limits()


def get_context_data(self, **kwargs):
    context = super(self.__class__, self).get_context_data(**kwargs)
    types = (
        ("totalSharesUsed", "maxTotalShares", _("Shares")),
        ("totalShareGigabytesUsed", "maxTotalShareGigabytes",
         _("Share Storage")),
        ("totalShareSnapshotsUsed", "maxTotalShareSnapshots",
         _("Share Snapshots")),
        ("totalSnapshotGigabytesUsed", "maxTotalSnapshotGigabytes",
         _("Share Snapshots Storage")),
        ("totalShareNetworksUsed", "maxTotalShareNetworks",
         _("Share Networks")),
    )
    for t in types:
        if t[0] in self.usage.limits and t[1] in self.usage.limits:
            context['charts'].append({
                'type': t[0],
                'name': t[2],
                'used': self.usage.limits[t[0]],
                'max': self.usage.limits[t[1]],
                'text': False,
            })
    return context



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.utils.translation import gettext_lazy as _
from horizon import workflows

from manila_ui.dashboards.project.share_networks.share_network_subnets \
    import workflows as subnet_workflows


class CreateSubnet(workflows.WorkflowView):
    workflow_class = subnet_workflows.CreateShareNetworkSubnetsWorkflow
    success_url = 'horizon:project:share_networks:index'
    page_title = _('Create Share Network Subnet')

    def get_initial(self):
        return {'share_network_id': self.kwargs["share_network_id"]}



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack_dashboard import api
from oslo_utils import timeutils
from unittest import mock

from manila_ui.api import manila as api_manila
from manila_ui.dashboards import utils
from manila_ui.tests.dashboards.project import test_data
from manila_ui.tests import helpers as test


class ShareNetworkSubnetsViewTests(test.TestCase):
    class FakeAZ(object):
        def __init__(self, name, id):
            self.name = name
            self.id = id
            self.created_at = timeutils.utcnow()

    def test_create_share_network_subnets(self):
        share_net = test_data.active_share_network
        network_id = share_net.id
        url = '/project/share_networks/' + network_id + '/subnets/create'
        neutron_net_id = self.networks.first().id
        sanitized_net_id = utils.transform_dashed_name(neutron_net_id)

        formData = {
            'method': 'CreateForm',
            'neutron_net_id': utils.transform_dashed_name(neutron_net_id),
            'availability_zone': 'fake_az',
            f'subnet-choices-{sanitized_net_id}':
                self.networks.first().subnets[0].id,
        }

        self.mock_object(
            api.neutron, "subnet_list",
            mock.Mock(return_value=self.subnets.list()))
        self.mock_object(
            api.neutron, "network_list",
            mock.Mock(return_value=self.networks.list()))
        self.mock_object(
            api_manila, "share_network_get",
            mock.Mock(return_value=mock.Mock(name='test_network'))
        )
        self.mock_object(
            api_manila, "share_network_subnet_create",
            mock.Mock(return_value=share_net))
        self.mock_object(
            api_manila, "availability_zone_list",
            mock.Mock(return_value=[self.FakeAZ('fake_az', 'fake_az')])
        )

        # Get response from create subnet url using formData values
        res = self.client.post(url, formData)

        self.assertNoFormErrors(res)
        self.assertMessageCount(error=0, warning=0)

        api_manila.availability_zone_list.assert_called_once_with(mock.ANY)
        api.neutron.network_list.assert_called_once_with(mock.ANY)
        api.neutron.subnet_list.assert_has_calls([
            mock.call(mock.ANY, network_id=network.id)
            for network in self.networks.list()
        ], any_order=True)
        api_manila.share_network_subnet_create.assert_called_once_with(
            mock.ANY,
            share_network_id=network_id,
            neutron_net_id=neutron_net_id,
            neutron_subnet_id=self.networks.first().subnets[0].id,
            availability_zone='fake_az')



# Copyright (c) 2020 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import tables
from horizon import tabs
from horizon.utils import memoized

from manila_ui.api import manila
from manila_ui.dashboards.project.user_messages import tables as um_tables
from manila_ui.dashboards.project.user_messages import tabs as um_tabs


class UserMessagesView(tables.MultiTableView):
    table_classes = (
        um_tables.UserMessagesTable,
    )
    template_name = "project/user_messages/index.html"
    page_title = _("User Messages")

    @memoized.memoized_method
    def get_user_messages_data(self):
        try:
            messages = manila.messages_list(self.request)
        except Exception:
            msg = _("Unable to retrieve messages list.")
            exceptions.handle(self.request, msg)
            return []
        return messages


class UserMessagesDetailView(tabs.TabView):
    tab_group_class = um_tabs.UserMessagesDetailTabs
    template_name = 'project/user_messages/detail.html'
    redirect_url = reverse_lazy('horizon:project:user_messages:index')

    def get_context_data(self, **kwargs):
        context = super(UserMessagesDetailView, self).get_context_data(
            **kwargs)
        message = self.get_data()
        message_id = message.id
        context["message"] = message
        context["message_id"] = message_id
        context["page_title"] = _("User Message Details: "
                                  "%(message_id)s") % (
            {'message_id': message_id})
        return context

    @memoized.memoized_method
    def get_data(self):
        try:
            message_id = self.kwargs['message_id']
            message = manila.messages_get(self.request, message_id)
        except Exception:
            exceptions.handle(
                self.request,
                _('Unable to retrieve user message detail.'),
                redirect=self.redirect_url)
        return message

    def get_tabs(self, request, *args, **kwargs):
        message = self.get_data()
        return self.tab_group_class(request, message=message, **kwargs)



# Copyright (c) 2014 NetApp, Inc.
# All rights reserved.

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Views for managing share snapshots.
"""

from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import messages

from manila_ui.api import manila


class CreateShareSnapshotForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length="255", label=_("Share Snapshot Name"))
    description = forms.CharField(
        widget=forms.Textarea,
        label=_("Description"), required=False)

    def __init__(self, request, *args, **kwargs):
        super(self.__class__, self).__init__(request, *args, **kwargs)
        # populate share_id
        share_id = kwargs.get('initial', {}).get('share_id', [])
        self.fields['share_id'] = forms.CharField(
            widget=forms.HiddenInput(), initial=share_id)

    def handle(self, request, data):
        try:
            snapshot = manila.share_snapshot_create(
                request, data['share_id'], data['name'], data['description'])
            message = _('Creating share snapshot "%s".') % data['name']
            messages.success(request, message)
            return snapshot
        except Exception:
            redirect = reverse("horizon:project:share_snapshots:index")
            exceptions.handle(request,
                              _('Unable to create share snapshot.'),
                              redirect=redirect)


class UpdateShareSnapshotForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length="255", label=_("Share Snapshot Name"))
    description = forms.CharField(
        widget=forms.Textarea, label=_("Description"), required=False)

    def handle(self, request, data):
        snapshot_id = self.initial['snapshot_id']
        try:
            manila.share_snapshot_update(
                request, snapshot_id, data['name'], data['description'])
            message = _('Updating share snapshot "%s"') % data['name']
            messages.success(request, message)
            return True
        except Exception:
            exceptions.handle(request, _('Unable to update share snapshot.'))


class AddShareSnapshotRule(forms.SelfHandlingForm):
    access_type = forms.ChoiceField(
        label=_("Access Type"), required=True,
        choices=(('ip', 'ip'), ('user', 'user'), ('cephx', 'cephx'),
                 ('cert', 'cert')))
    access_to = forms.CharField(
        label=_("Access To"), max_length="255", required=True)

    def handle(self, request, data):
        snapshot_id = self.initial['snapshot_id']
        try:
            manila.share_snapshot_allow(
                request, snapshot_id,
                access_to=data['access_to'],
                access_type=data['access_type'])
            message = _('Creating snapshot rule for "%s"') % data['access_to']
            messages.success(request, message)
            return True
        except Exception:
            redirect = reverse(
                "horizon:project:share_snapshots:share_snapshot_manage_rules",
                args=[self.initial['snapshot_id']])
            exceptions.handle(
                request, _('Unable to add snapshot rule.'), redirect=redirect)



# Copyright 2017 Mirantis, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import workflows
from openstack_dashboard.api import keystone

from manila_ui.api import manila


class AddProjectAction(workflows.MembershipAction):

    def __init__(self, request, *args, **kwargs):
        super(AddProjectAction, self).__init__(request, *args, **kwargs)
        default_role_field_name = self.get_default_role_field_name()
        self.fields[default_role_field_name] = forms.CharField(required=False)
        self.fields[default_role_field_name].initial = 'member'

        field_name = self.get_member_field_name('member')
        self.fields[field_name] = forms.MultipleChoiceField(required=False)
        share_group_type_id = self.initial['id']

        # Get list of existing projects
        try:
            projects, __ = keystone.tenant_list(request)
        except Exception:
            err_msg = _('Unable to get list of projects.')
            exceptions.handle(request, err_msg)

        # Get list of projects with access to this Share Group Type
        try:
            share_group_type = manila.share_group_type_get(
                request, share_group_type_id)
            self.share_group_type_name = share_group_type.name
            projects_initial = manila.share_group_type_access_list(
                request, share_group_type)
        except Exception:
            err_msg = _(
                'Unable to get information about share group type access.')
            exceptions.handle(request, err_msg)

        self.fields[field_name].choices = [
            (project.id, project.name or project.id) for project in projects]
        self.fields[field_name].initial = [
            pr.project_id for pr in projects_initial]
        self.projects_initial = set(self.fields[field_name].initial)

    class Meta(object):
        name = _("Projects with access to share group type")
        slug = "update_members"

    def handle(self, request, context):
        context.update({
            'name': self.share_group_type_name,
            'projects_add': self.projects_allow - self.projects_initial,
            'projects_remove': self.projects_initial - self.projects_allow,
        })
        return context

    def clean(self):
        cleaned_data = super(AddProjectAction, self).clean()
        self.projects_allow = set(
            cleaned_data[self.get_member_field_name('member')])
        return cleaned_data


class AddProjectStep(workflows.UpdateMembersStep):
    action_class = AddProjectAction
    available_list_title = _("Available projects")
    help_text = _("Allow project access to share group type.")
    members_list_title = _("Selected projects")
    no_available_text = _("No projects found.")
    no_members_text = _("No projects selected.")
    depends_on = ("id", )
    show_roles = False


class ManageShareGroupTypeAccessWorkflow(workflows.Workflow):
    slug = "manage_share_group_type_access"
    name = _("Manage Share Group Type Access")
    finalize_button_name = _("Manage Share Group Type Access")
    success_message = _('Updated access for share group type "%s".')
    failure_message = _('Unable to update access for share group type "%s".')
    success_url = 'horizon:admin:share_group_types:index'
    default_steps = (AddProjectStep, )

    def format_status_message(self, message):
        return message % self.context['name']

    def handle(self, request, context):
        try:
            for project in self.context['projects_remove']:
                manila.share_group_type_access_remove(
                    request, self.context['id'], project)
            for project in self.context['projects_add']:
                manila.share_group_type_access_add(
                    request, self.context['id'], project)
            return True
        except Exception:
            exceptions.handle(request, _('Unable to update share group type.'))
            return False



# Copyright (c) 2014 NetApp, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manilaclient.v2 import messages
from manilaclient.v2 import security_services
from manilaclient.v2 import share_access_rules
from manilaclient.v2 import share_export_locations
from manilaclient.v2 import share_group_snapshots
from manilaclient.v2 import share_group_types
from manilaclient.v2 import share_groups
from manilaclient.v2 import share_instances
from manilaclient.v2 import share_networks
from manilaclient.v2 import share_replicas
from manilaclient.v2 import share_servers
from manilaclient.v2 import share_snapshot_export_locations
from manilaclient.v2 import share_snapshots
from manilaclient.v2 import share_types
from manilaclient.v2 import shares


class FakeAPIClient(object):
    client = "fake_client"


share = shares.Share(
    shares.ShareManager(FakeAPIClient),
    {'id': "11023e92-8008-4c8b-8059-7f2293ff3887",
     'status': 'available',
     'size': 40,
     'name': 'Share name',
     'description': 'Share description',
     'is_public': False,
     'share_proto': 'NFS',
     'metadata': {},
     'created_at': '2014-01-27 10:30:00',
     'share_server_id': '1',
     'share_network_id': '7f3d1c33-8d00-4511-29df-a2def31f3b5d',
     'availability_zone': 'Test AZ',
     'replication_type': 'readable',
     'share_group_id': 'fake_share_group_id',
     'mount_snapshot_support': False})

nameless_share = shares.Share(
    shares.ShareManager(FakeAPIClient),
    {'id': "4b069dd0-6eaa-4272-8abc-5448a68f1cce",
     'status': 'available',
     'size': 10,
     'name': '',
     'description': '',
     'share_proto': 'NFS',
     'export_location': "/dev/hda",
     'metadata': {},
     'created_at': '2010-11-21 18:34:25',
     'share_type': 'vol_type_1',
     'share_server_id': '1',
     'share_network_id': '7f3d1c33-8d00-4511-29df-a2def31f3b5d',
     'availability_zone': 'Test AZ',
     'replication_type': None,
     'mount_snapshot_support': False})

share_with_metadata = shares.Share(
    shares.ShareManager(FakeAPIClient),
    {'id': "0ebb3748-c1dr-4bb6-8315-0354e7691fff",
     'status': 'available',
     'size': 40,
     'name': 'Share with metadata',
     'description': 'Share description',
     'share_proto': 'NFS',
     'metadata': {'aaa': 'bbb'},
     'created_at': '2016-06-31 00:00:00',
     'share_server_id': '1',
     'share_network_id': '7f3d1c33-8d00-4511-29df-a2def31f3b5d',
     'availability_zone': 'Test AZ',
     'replication_type': 'readable',
     'mount_snapshot_support': False})

other_share = shares.Share(
    shares.ShareManager(FakeAPIClient),
    {'id': "21023e92-8008-1234-8059-7f2293ff3889",
     'status': 'in-use',
     'size': 10,
     'name': 'my_share',
     'description': '',
     'share_proto': 'NFS',
     'metadata': {},
     'created_at': '2013-04-01 10:30:00',
     'share_type': None,
     'share_server_id': '1',
     'share_network_id': '7f3d1c33-8d00-4511-29df-a2def31f3b5d',
     'availability_zone': 'Test AZ',
     'replication_type': 'readable',
     'mount_snapshot_support': False})

shares_list = [share, nameless_share, other_share]

share_replica = share_replicas.ShareReplica(
    share_replicas.ShareReplicaManager(FakeAPIClient),
    {'id': '11023e92-8008-4c8b-8059-replica00001',
     'availability_zone': share.availability_zone,
     'host': 'fake_host_1',
     'share_id': share.id,
     'status': 'available',
     'replica_state': 'active',
     'created_at': '2016-07-19 19:46:13',
     'updated_at': '2016-07-19 19:47:14'}
)

share_replica2 = share_replicas.ShareReplica(
    share_replicas.ShareReplicaManager(FakeAPIClient),
    {'id': '11023e92-8008-4c8b-8059-replica00002',
     'availability_zone': share.availability_zone,
     'host': 'fake_host_2',
     'share_id': share.id,
     'status': 'available',
     'replica_state': 'in_sync',
     'created_at': '2016-07-19 20:46:13',
     'updated_at': '2016-07-19 20:47:14'}
)

share_replica3 = share_replicas.ShareReplica(
    share_replicas.ShareReplicaManager(FakeAPIClient),
    {'id': '11023e92-8008-4c8b-8059-replica00003',
     'availability_zone': share.availability_zone,
     'host': 'fake_host_3',
     'share_id': share.id,
     'status': 'available',
     'replica_state': 'active',
     'created_at': '2016-07-19 21:46:13',
     'updated_at': '2016-07-19 21:47:14'}
)

share_mount_snapshot = shares.Share(
    shares.ShareManager(FakeAPIClient),
    {'id': "11023e92-8008-4c8b-8059-7f2293ff3888",
     'status': 'available',
     'size': 40,
     'name': 'Share name',
     'description': 'Share description',
     'share_proto': 'NFS',
     'metadata': {},
     'created_at': '2014-01-27 10:30:00',
     'share_server_id': '1',
     'share_network_id': '7f3d1c33-8d00-4511-29df-a2def31f3b5d',
     'availability_zone': 'Test AZ',
     'replication_type': 'readable',
     'mount_snapshot_support': True})

admin_export_location = share_export_locations.ShareExportLocation(
    share_export_locations.ShareExportLocationManager(FakeAPIClient),
    {'id': '6921e862-88bc-49a5-a2df-efeed9acd583',
     'path': '1.1.1.1:/path/to/admin/share',
     'preferred': False,
     'is_admin_only': True,
     'share_instance_id': 'e1c2d35e-fe67-4028-ad7a-45f668732b1d'}
)

user_export_location = share_export_locations.ShareExportLocation(
    share_export_locations.ShareExportLocationManager(FakeAPIClient),
    {'id': 'b6bd76ce-12a2-42a9-a30a-8a43b503867d',
     'path': '2.2.2.2:/path/to/user/share',
     'preferred': True,
     'is_admin_only': False,
     'share_instance_id': 'e1c2d35e-fe67-4028-ad7a-45f668732b1d'}
)

export_locations = [admin_export_location, user_export_location]

admin_snapshot_export_locations = [
    share_snapshot_export_locations.ShareSnapshotExportLocation(
        share_snapshot_export_locations.ShareSnapshotExportLocationManager(
            FakeAPIClient),
        {'id': '6921e862-88bc-49a5-a2df-efeed9acd584',
         'path': '1.1.1.1:/path/to/admin/share',
         'is_admin_only': True,
         'share_snapshot_instance_id': 'e1c2d35e-fe67-4028-ad7a-45f668732b1e'}
    ),
    share_snapshot_export_locations.ShareSnapshotExportLocation(
        share_snapshot_export_locations.ShareSnapshotExportLocationManager(
            FakeAPIClient),
        {'id': '6921e862-88bc-49a5-a2df-efeed9acd585',
         'path': '1.1.1.2:/path/to/admin/share',
         'is_admin_only': False,
         'share_snapshot_instance_id': 'e1c2d35e-fe67-4028-ad7a-45f668732b1f'}
    )
]

user_snapshot_export_locations = [
    share_snapshot_export_locations.ShareSnapshotExportLocation(
        share_snapshot_export_locations.ShareSnapshotExportLocationManager(
            FakeAPIClient),
        {'id': 'b6bd76ce-12a2-42a9-a30a-8a43b503867e',
         'path': '1.1.1.1:/path/to/user/share_snapshot'}
    ),
    share_snapshot_export_locations.ShareSnapshotExportLocation(
        share_snapshot_export_locations.ShareSnapshotExportLocationManager(
            FakeAPIClient),
        {'id': 'b6bd76ce-12a2-42a9-a30a-8a43b503867f',
         'path': '1.1.1.2:/not/too/long/path/to/user/share_snapshot'}
    )
]
ip_rule = share_access_rules.ShareAccessRule(
    share_access_rules.ShareAccessRuleManager(FakeAPIClient),
    {'id': 'ca8b755c-fe13-497f-81d0-fd2f13a30a78',
     'access_level': 'rw',
     'access_to': '1.1.1.1',
     'access_type': 'ip',
     'state': 'active',
     'access_key': '',
     'created_at': '2021-03-03T14:29:41.000000',
     'updated_at': '',
     "metadata": {},
     })

user_rule = share_access_rules.ShareAccessRule(
    share_access_rules.ShareAccessRuleManager(FakeAPIClient),
    {'id': '0837072-c49e-11e3-bd64-60a44c371189',
     'access_level': 'rw',
     'access_to': 'someuser',
     'access_type': 'user',
     'state': 'active',
     'access_key': '',
     'created_at': '2021-03-03T14:29:41.000000',
     'updated_at': '',
     'metadata': {'abc': 'ddd'},
     })

cephx_rule = share_access_rules.ShareAccessRule(
    share_access_rules.ShareAccessRuleManager(FakeAPIClient),
    {'id': '235481bc-1a84-11e6-9666-68f728a0492e',
     'access_level': 'rw',
     'access_to': 'alice',
     'access_type': 'cephx',
     'state': 'active',
     'access_key': 'AQAdFCNYDCapMRAANuK/CiEZbog2911a+t5dcQ==',
     'created_at': '2021-03-03T14:29:41.000000',
     'updated_at': '',
     'metadata': {'test': 'true'},
     })

share_access_list = [user_rule, cephx_rule, ip_rule]

snapshot = share_snapshots.ShareSnapshot(
    share_snapshots.ShareSnapshotManager(FakeAPIClient),
    {'id': '5f3d1c33-7d00-4511-99df-a2def31f3b5d',
     'name': 'test snapshot',
     'description': 'share snapshot',
     'size': 40,
     'status': 'available',
     'share_id': '11023e92-8008-4c8b-8059-7f2293ff3887'})

snapshot_mount_support = share_snapshots.ShareSnapshot(
    share_snapshots.ShareSnapshotManager(FakeAPIClient),
    {'id': '5f3d1c33-7d00-4511-99df-a2def31f3b5e',
     'name': 'test snapshot',
     'description': 'share snapshot',
     'size': 40,
     'status': 'available',
     'share_id': '11023e92-8008-4c8b-8059-7f2293ff3888'})

inactive_share_network = share_networks.ShareNetwork(
    share_networks.ShareNetworkManager(FakeAPIClient),
    {'id': '6f3d1c33-8d00-4511-29df-a2def31f3b5d',
     'name': 'test_share_net',
     'description': 'test share network',
     'status': 'INACTIVE',
     'neutron_net_id': 'fake_neutron_net_id',
     'neutron_subnet_id': 'fake_neutron_subnet_id'})

active_share_network = share_networks.ShareNetwork(
    share_networks.ShareNetworkManager(FakeAPIClient),
    {
        "id": "1324e7d3-fba8-45e4-bb37-b59c12eb06dc",
        "name": "net_my1",
        "project_id": "16e1ab15c35a457e9c2b2aa189f544e1",
        "created_at": "2019-10-02T17:49:43.000000",
        "description": "This is test share network",
        "security_service_update_support": True,
        "status": "active",
        "share_network_subnets": [
            {
                "id": "e4db03dc-6041-4c6a-a8f9-80bb4141a1eb",
                "availability_zone": None,
                "created_at": "2019-10-02T17:49:43.000000",
                "updated_at": "2019-10-03T12:17:39.000000",
                "segmentation_id": None,
                "neutron_net_id": "62187648-6617-4509-a780-ffc973a7fe43",
                "neutron_subnet_id": "2276888a-27c1-47c2-82a0-ea33050128b5",
                "ip_version": 4,
                "cidr": "172.24.5.0/24",
                "network_type": "flat",
                "mtu": 1500,
                "gateway": "172.24.5.1",
            },
            {
                "id": "e4db03dc-6041-4c6a-a8f9-80bb4141a1en",
                "availability_zone": "manila-zone-0",
                "created_at": "2019-10-02T17:49:43.000000",
                "updated_at": "2019-10-03T12:17:39.000000",
                "segmentation_id": None,
                "neutron_net_id": "62187648-6617-4509-a780-ffc973a7f333",
                "neutron_subnet_id": "2276888a-27c1-47c2-82a0-ea3305012905",
                "ip_version": 4,
                "cidr": "172.24.5.0/24",
                "network_type": "flat",
                "mtu": 1500,
                "gateway": "172.24.5.1",
            },
        ],
    }
)

sec_service = security_services.SecurityService(
    security_services.SecurityServiceManager(FakeAPIClient),
    {'id': '7f3d1c33-8d10-4511-29df-a2def31f3b5d',
     'server': '1.1.1.1',
     'dns_ip': '2.2.2.2',
     'ou': 'someOU',
     'user': 'someuser',
     'password': 'somepass',
     'type': 'active_directory',
     'name': 'test-sec-service',
     'description': 'test security service',
     'domain': 'testdomain',
     })


share_instance = share_instances.ShareInstance(
    share_instances.ShareInstanceManager(FakeAPIClient),
    {'id': 'fake_share_instance_no_ss_id',
     'status': 'available',
     'host': 'host1@backend1#pool1',
     'availability_zone': 'zone1',
     'share_id': 'fake_share_id_1',
     'share_network_id': 'fake_share_network_id_1',
     'share_server_id': 'fake_share_server_id_1',
     'created_at': '2016-04-26 13:14:15'}
)


share_instance_no_ss = share_instances.ShareInstance(
    share_instances.ShareInstanceManager(FakeAPIClient),
    {'id': 'fake_share_instance_id',
     'status': 'available',
     'host': 'host2@backend2#pool2',
     'availability_zone': 'zone2',
     'share_id': 'fake_share_id_2',
     'share_network_id': None,
     'share_server_id': None,
     'created_at': '2016-04-26 14:15:16'}
)

share_server = share_servers.ShareServer(
    share_servers.ShareServerManager(FakeAPIClient),
    {'id': 'fake_share_server_id1',
     'status': 'active',
     'share_network_id': 'fake_share_network_id1',
     'share_network_name': 'fake_share_network_name1',
     'project_id': 'fake_project_id1',
     'backend_details': {
         'foo_key': 'foo_value',
         'bar_key_foo': 'bar_value_foo',
     },
     'host': 'fakehost1@fakebackend1#fakepool1'}
)

share_server_errored = share_servers.ShareServer(
    share_servers.ShareServerManager(FakeAPIClient),
    {'id': 'fake_share_server_id2',
     'status': 'error',
     'share_network_id': 'fake_share_network_id2',
     'share_network_name': 'fake_share_network_name2',
     'project_id': 'fake_project_id2',
     'backend_details': {},
     'host': 'fakehost2@fakebackend2#fakepool2'}
)

share_type = share_types.ShareType(
    share_types.ShareTypeManager(FakeAPIClient),
    {'id': 'share-type-id1',
     'name': 'test-share-type1',
     'share_type_access:is_public': True,
     'extra_specs': {
         'snapshot_support': True,
         'driver_handles_share_servers': False}
     }
)

share_type_private = share_types.ShareType(
    share_types.ShareTypeManager(FakeAPIClient),
    {'id': 'share-type-id2',
     'name': 'test-share-type2',
     'share_type_access:is_public': False,
     'extra_specs': {'driver_handles_share_servers': False}}
)

share_type_dhss_true = share_types.ShareType(
    share_types.ShareTypeManager(FakeAPIClient),
    {'id': 'share-type-id3',
     'name': 'test-share-type3',
     'share_type_access:is_public': True,
     'extra_specs': {'driver_handles_share_servers': True}}
)

share_type_alt = share_types.ShareType(
    share_types.ShareTypeManager(FakeAPIClient),
    {'id': 'share-type-id4',
     'name': 'test-share-type4',
     'share_type_access:is_public': True,
     'extra_specs': {
         'snapshot_support': True,
         'driver_handles_share_servers': False}
     }
)

share_group_type = share_group_types.ShareGroupType(
    share_group_types.ShareGroupTypeManager(FakeAPIClient),
    {'id': 'fake_share_group_type_id1',
     'name': 'fake_share_group_type_name',
     'share_types': [share_type.id],
     'group_specs': {'k1': 'v1', 'k2': 'v2'},
     'is_public': True}
)

share_group_type_private = share_group_types.ShareGroupType(
    share_group_types.ShareGroupTypeManager(FakeAPIClient),
    {'id': 'fake_private_share_group_type_id2',
     'name': 'fake_private_share_group_type_name',
     'share_types': [share_type.id, share_type_private.id],
     'group_specs': {'k1': 'v1', 'k2': 'v2'},
     'is_public': False}
)

share_group_type_dhss_true = share_group_types.ShareGroupType(
    share_group_types.ShareGroupTypeManager(FakeAPIClient),
    {'id': 'fake_share_group_type_id3',
     'name': 'fake_share_group_type_name',
     'share_types': [share_type_dhss_true.id],
     'group_specs': {'k3': 'v3', 'k4': 'v4'},
     'is_public': True}
)

share_group_type_alt = share_group_types.ShareGroupType(
    share_group_types.ShareGroupTypeManager(FakeAPIClient),
    {'id': 'fake_share_group_type_id4',
     'name': 'fake_share_group_type_name',
     'share_types': [share_type_alt.id],
     'group_specs': {'k5': 'v5', 'k6': 'v6'},
     'is_public': True}
)

share_group = share_groups.ShareGroup(
    share_groups.ShareGroupManager(FakeAPIClient),
    {'id': 'fake_share_group_id',
     'name': 'fake_share_group_name',
     'description': 'fake sg description',
     'status': 'available',
     'share_types': [share_type.id],
     'share_group_type_id': share_group_type.id,
     'source_share_group_snapshot_id': None,
     'share_network_id': None,
     'share_server_id': None,
     'availability_zone': None,
     'host': 'fake_host_987654321',
     'consistent_snapshot_support': None,
     'created_at': '2017-05-31T13:36:15.000000',
     'project_id': 'fake_project_id_987654321'}
)

share_group_nameless = share_groups.ShareGroup(
    share_groups.ShareGroupManager(FakeAPIClient),
    {'id': 'fake_nameless_share_group_id',
     'name': None,
     'status': 'available',
     'share_types': [share_type.id],
     'share_group_type_id': share_group_type.id,
     'source_share_group_snapshot_id': None,
     'share_network_id': None,
     'share_server_id': None,
     'availability_zone': None,
     'host': 'fake_host_987654321',
     'consistent_snapshot_support': None,
     'created_at': '2017-05-31T13:36:15.000000',
     'project_id': 'fake_project_id_987654321'}
)

share_group_dhss_true = share_groups.ShareGroup(
    share_groups.ShareGroupManager(FakeAPIClient),
    {'id': 'fake_dhss_true_share_group_id',
     'name': 'fake_dhss_true_share_group_name',
     'status': 'available',
     'share_types': [share_type_dhss_true.id],
     'share_group_type_id': share_group_type_dhss_true.id,
     'source_share_group_snapshot_id': None,
     'share_network_id': 'fake_share_network_id',
     'share_server_id': 'fake_share_server_id',
     'availability_zone': None,
     'host': 'fake_host_987654321',
     'consistent_snapshot_support': 'pool',
     'created_at': '2017-05-31T23:59:59.000000',
     'project_id': 'fake_project_id_987654321'}
)

share_group_snapshot = share_group_snapshots.ShareGroupSnapshot(
    share_group_snapshots.ShareGroupSnapshotManager(FakeAPIClient),
    {'id': 'fake_share_group_snapshot_id_1',
     'name': 'fake_share_group_snapshot_name',
     'status': 'available',
     'share_group_id': share_group.id,
     'description': 'fake sgs description',
     'created_at': '2017-06-01T13:13:13.000000',
     'project_id': 'fake_project_id_987654321',
     'members': [
         {'share_id': 'fake_share_id_1', 'id': 'fake_ssi_id_1', 'size': 1},
         {'share_id': 'fake_share_id_2', 'id': 'fake_ssi_id_2', 'size': 2},
     ]}
)

share_group_snapshot_nameless = share_group_snapshots.ShareGroupSnapshot(
    share_group_snapshots.ShareGroupSnapshotManager(FakeAPIClient),
    {'id': 'fake_share_group_snapshot_id_2_nameless',
     'name': None,
     'status': 'available',
     'share_group_id': share_group_nameless.id,
     'description': 'fake nameless sgs description',
     'created_at': '2017-06-02T14:14:14.000000',
     'project_id': 'fake_project_id_987654321',
     'members': []}
)

# Manila Limits
limits = {"totalSharesUsed": 1,
          "totalShareSnapshotsUsed": 1,
          "totalShareGigabytesUsed": 500,
          "totalSnapshotGigabytesUsed": 500,
          "maxTotalShares": 10,
          "maxTotalShareSnapshots": 10,
          "maxTotalShareGigabytes": 1000,
          "maxTotalSnapshotGigabytes": 1000,
          }

limits_negative = {"totalSharesUsed": 10,
                   "totalShareSnapshotsUsed": 10,
                   "totalShareGigabytesUsed": 1000,
                   "totalSnapshotGigabytesUsed": 1000,
                   "maxTotalShares": 10,
                   "maxTotalShareSnapshots": 10,
                   "maxTotalShareGigabytes": 1000,
                   "maxTotalSnapshotGigabytes": 1000,
                   }

# Manila User Messages
fake_message_1 = messages.Message(
    messages.MessageManager(FakeAPIClient),
    {'resource_id': "351cc796-2d79-4a08-b878-a8ed933b6b68",
     'message_level': 'ERROR',
     'user_message': 'allocate host: No storage could be allocated for'
                     ' this share request. Trying again with a different'
                     ' size or share type may succeed.',
     'expires_at': '2017-07-10T10:27:43.000000',
     'id': '4b319d29-d5b7-4b6e-8e7c-8d6e53f3c3d5',
     'created_at': '2017-07-10T10:26:43.000000',
     'detail_id': '002',
     'request_id': 'req-24e7ccb6-a7d5-4ddd-a8e4-d8f72a4509c8',
     'project_id': '2e3de76b49b444fd9dc7ca9f7048ce6b',
     'resource_type': 'SHARE',
     'action_id': '001'})

fake_message_2 = messages.Message(
    messages.MessageManager(FakeAPIClient),
    {'resource_id': "25b4c0cc-e711-4c6f-b9fd-72d6b5c62bce",
     'message_level': 'ERROR',
     'user_message': 'Driver does not expect share-network to be provided '
                     'with current configuration.',
     'expires_at': '2018-09-10T09:37:45.000000',
     'id': 'd01d03ee-7758-4175-a6b5-853329dd2f4e',
     'created_at': '2018-09-10T09:36:45.000000',
     'detail_id': '003',
     'request_id': 'req-fa568ab0-d6b3-4b32-899d-637ee006fed4',
     'project_id': '2e3de76b49b444fd9dc7ca9f7048ce6b',
     'resource_type': 'SHARE',
     'action_id': '002'})

fake_message_3 = messages.Message(
    messages.MessageManager(FakeAPIClient),
    {'resource_id': "4befdc84-2796-44e1-8645-14b651bfb787",
     'message_level': 'ERROR',
     'user_message': 'Share has no replica with "replica_state" '
                     'set to "active"',
     'expires_at': '2020-09-09T22:37:13.000000',
     'id': '1e1c493f-d07d-48e9-93a8-ef5ad4b8ca8a',
     'created_at': '2020-09-09T22:36:13.000000',
     'detail_id': '004',
     'request_id': 'req-29449bc8-d0ec-4d6b-b37c-85d0f04251b1',
     'project_id': '2e3de76b49b444fd9dc7ca9f7048ce6b',
     'resource_type': 'SHARE_REPLICA',
     'action_id': '002'})



# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import tables
from horizon import tabs
from horizon.utils import memoized
from horizon import workflows
from openstack_dashboard.api import base
from openstack_dashboard.api import neutron

from manila_ui.api import manila
from manila_ui.dashboards.project.share_networks import tables as sn_tables
from manila_ui.dashboards.project.share_networks import tabs as sn_tabs
import manila_ui.dashboards.project.share_networks.workflows as sn_workflows
from manila_ui.dashboards import utils


class ShareNetworksView(tables.MultiTableView):
    table_classes = (
        sn_tables.ShareNetworksTable,
    )
    template_name = "admin/share_networks/index.html"
    page_title = _("Share Networks")

    @memoized.memoized_method
    def get_share_networks_data(self):
        try:
            share_networks = manila.share_network_list(
                self.request, detailed=True)
        except Exception:
            share_networks = []
            exceptions.handle(
                self.request, _("Unable to retrieve share networks"))
        share_networks = self.get_filters(share_networks)
        return share_networks

    def get_filters(self, share_networks):
        table = self._tables['share_networks']
        filters = self.get_server_filter_info(table.request, table)
        filter_string = filters['value']
        filter_field = filters['field']
        if filter_string and filter_field:
            filtered_data = []
            for share_network in share_networks:
                if filter_field == 'name':
                    if share_network.name == filter_string:
                        filtered_data.append(share_network)

                if filter_field == 'description':
                    re_string = re.compile(filter_string)
                    if (share_network.description and
                            re.search(re_string,
                                      share_network.description)):
                        filtered_data.append(share_network)
            return filtered_data
        else:
            return share_networks


class Update(workflows.WorkflowView):
    workflow_class = sn_workflows.UpdateShareNetworkWorkflow
    template_name = "project/share_networks/update.html"
    success_url = 'horizon:project:share_networks:index'
    page_title = _('Update Share Network')

    def get_initial(self):
        return {'id': self.kwargs["share_network_id"]}

    def get_context_data(self, **kwargs):
        context = super(Update, self).get_context_data(**kwargs)
        context['id'] = self.kwargs['share_network_id']
        return context


class Create(workflows.WorkflowView):
    workflow_class = sn_workflows.CreateShareNetworkWorkflow
    form_id = "create_share_network"
    submit_url = reverse_lazy(
        "horizon:project:share_networks:share_network_create")
    success_url = reverse_lazy("horizon:project:share_networks:index")
    page_title = _('Create Share Network')


class Detail(tabs.TabView):
    tab_group_class = sn_tabs.ShareNetworkDetailTabs
    template_name = 'project/share_networks/detail.html'
    redirect_url = reverse_lazy('horizon:project:share_networks:index')

    def get_context_data(self, **kwargs):
        context = super(Detail, self).get_context_data(**kwargs)
        share_network = self.get_data()
        share_network_display_name = share_network.name or share_network.id
        context["share_network"] = share_network
        context["share_network_display_name"] = share_network_display_name
        context["page_title"] = _("Share Network Details: "
                                  "%(network_display_name)s") % \
            {'network_display_name': share_network_display_name}
        return context

    @memoized.memoized_method
    def get_data(self):
        try:
            share_net_id = self.kwargs['share_network_id']
            share_net = manila.share_network_get(self.request, share_net_id)
            if base.is_service_enabled(self.request, 'network'):
                for subnet in share_net.share_network_subnets:
                    # Neutron Net ID
                    try:
                        subnet["neutron_net"] = neutron.network_get(
                            self.request, subnet["neutron_net_id"]).name_or_id
                    except (
                        neutron.neutron_client.exceptions
                        .NeutronClientException
                        ):
                        subnet["neutron_net"] = _("Unknown")
                    # Neutron Subnet ID
                    try:
                        subnet["neutron_subnet"] = neutron.subnet_get(
                            self.request,
                            subnet["neutron_subnet_id"]).name_or_id
                    except (
                        neutron.neutron_client.exceptions
                        .NeutronClientException
                        ):
                        subnet["neutron_subnet"] = _("Unknown")
            # List all azs if availability_zone is None
            availability_zones = manila.availability_zone_list(self.request)
            az_list = ", ".join([az.name for az in availability_zones])
            for subnet in share_net.share_network_subnets:
                if subnet["availability_zone"] is None:
                    subnet["availability_zone"] = az_list
            share_net.sec_services = (
                manila.share_network_security_service_list(
                    self.request, share_net_id))
            for ss in share_net.sec_services:
                ss.type = utils.get_nice_security_service_type(ss)
            server_search_opts = {'share_network_id': share_net_id}
            try:
                share_servs = manila.share_server_list(
                    self.request,
                    search_opts=server_search_opts)
            #  Non admins won't be able to get share servers
            except Exception:
                share_servs = []
            if share_servs:
                share_net.share_servers = share_servs
        except Exception:
            exceptions.handle(self.request,
                              _('Unable to retrieve share network details.'),
                              redirect=self.redirect_url)
        return share_net

    def get_tabs(self, request, *args, **kwargs):
        share_network = self.get_data()
        return self.tab_group_class(request, share_network=share_network,
                                    **kwargs)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import workflows

from openstack_dashboard.api import base
from openstack_dashboard.dashboards.identity.projects \
    import workflows as project_workflows

from manila_ui.api import manila as api_manila

LOG = logging.getLogger(__name__)


class ShareQuotaAction(project_workflows.CommonQuotaAction):
    shares = forms.IntegerField(min_value=-1, label=_("Shares"))
    share_gigabytes = forms.IntegerField(
        min_value=-1, label=_("Share gigabytes"))
    share_snapshots = forms.IntegerField(
        min_value=-1, label=_("Share snapshots"))
    share_snapshot_gigabytes = forms.IntegerField(
        min_value=-1, label=_("Share snapshot gigabytes"))
    share_networks = forms.IntegerField(
        min_value=-1, label=_("Share Networks"))
    share_groups = forms.IntegerField(
        min_value=-1, label=_("Share Groups"))
    share_group_snapshots = forms.IntegerField(
        min_value=-1, label=_("Share Group Snapshots"))

    _quota_fields = api_manila.MANILA_QUOTA_FIELDS

    def _tenant_quota_update(self, request, project_id, data):
        api_manila.tenant_quota_update(request, project_id, **data)

    class Meta(object):
        name = _("Share")
        slug = 'update_share_quotas'
        help_text = _("Set maximum quotas for the project.")
        permissions = ('openstack.roles.admin', 'openstack.services.share')


class UpdateShareQuota(workflows.Step):
    action_class = ShareQuotaAction
    depends_on = ("project_id", "disabled_quotas")
    contributes = api_manila.MANILA_QUOTA_FIELDS

    def prepare_action_context(self, request, context):
        try:
            quotas = api_manila.tenant_quota_get(
                request, context['project_id'])
            for field in api_manila.MANILA_QUOTA_FIELDS:
                # Resolve mismatch UI field names and data field names.
                data_field = api_manila.MANILA_QUOTA_FIELDS_DATA_MAP[field]
                context[field] = quotas.get(data_field).limit
        except Exception as ex:
            LOG.exception(ex)
            exceptions.handle(request,
                              _('Unable to retrieve share quotas.'))
        return context

    def allowed(self, request):
        return base.is_service_enabled(request, 'share')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy
from horizon import tables

from manila_ui.api import manila


class DeleteSecurityService(tables.DeleteAction):
    policy_rules = (("share", "security_service:delete"),)

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Delete Security Service",
            u"Delete Security Services",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Deleted Security Service",
            u"Deleted Security Services",
            count
        )

    def delete(self, request, obj_id):
        manila.security_service_delete(request, obj_id)


class SecurityServicesTable(tables.DataTable):
    name = tables.WrappingColumn(
        "name", verbose_name=_("Name"),
        link="horizon:admin:security_services:security_service_detail")
    description = tables.Column("description", verbose_name=_("Description"))
    project = tables.Column("project_name", verbose_name=_("Project"))
    dns_ip = tables.Column("dns_ip", verbose_name=_("DNS IP"))
    ou = tables.Column("ou", verbose_name=_("Organizational Unit"))
    server = tables.Column("server", verbose_name=_("Server"))
    domain = tables.Column("domain", verbose_name=_("Domain"))
    user = tables.Column("user", verbose_name=_("Sid"))

    def get_object_display(self, security_service):
        return security_service.name

    def get_object_id(self, security_service):
        return str(security_service.id)

    class Meta(object):
        name = "security_services"
        verbose_name = _("Security Services")
        table_actions = (
            tables.NameFilterAction,
            DeleteSecurityService,
        )
        row_actions = (
            DeleteSecurityService,
        )



# Copyright 2017 Mirantis, Inc.
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.template.defaultfilters import title
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy
from django.utils.translation import pgettext_lazy
from horizon import exceptions
from horizon import tables
from horizon.utils import filters

from manila_ui.api import manila


class ShareGroupSnapshotShareGroupNameColumn(tables.Column):
    def get_link_url(self, snapshot):
        return reverse(self.link, args=(snapshot.share_group_id,))


class DeleteShareGroupSnapshot(tables.DeleteAction):

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Delete Share Group Snapshot",
            u"Delete Share Group Snapshots",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Deleted Share Group Snapshot",
            u"Deleted Share Group Snapshots",
            count
        )

    def delete(self, request, obj_id):
        obj = self.table.get_object_by_id(obj_id)
        name = self.table.get_object_display(obj)
        try:
            manila.share_group_snapshot_delete(request, obj_id)
        except Exception:
            msg = _('Unable to delete share group snapshot "%s". '
                    'One or more share groups depend on it.')
            exceptions.handle(self.request, msg % name)
            raise

    def allowed(self, request, snapshot=None):
        if snapshot:
            return snapshot.status.lower() in ('available', 'error')
        return True


class ResetShareGroupSnapshotStatus(tables.LinkAction):
    name = "reset_share_group_snapshot_status"
    verbose_name = _("Reset status")
    url = "horizon:admin:share_group_snapshots:reset_status"
    classes = ("ajax-modal", "btn-create")

    def allowed(self, request, share_group=None):
        return True


class UpdateShareGroupSnapshotRow(tables.Row):
    ajax = True

    def get_data(self, request, share_group_snapshot_id):
        snapshot = manila.share_group_snapshot_get(
            request, share_group_snapshot_id)
        if not snapshot.name:
            snapshot.name = share_group_snapshot_id
        return snapshot


class ShareGroupSnapshotsTable(tables.DataTable):
    STATUS_CHOICES = (
        ("available", True),
        ("creating", None),
        ("error", False),
    )
    STATUS_DISPLAY_CHOICES = (
        ("available",
         pgettext_lazy("Current status of snapshot", u"Available")),
        ("creating", pgettext_lazy("Current status of snapshot", u"Creating")),
        ("error", pgettext_lazy("Current status of snapshot", u"Error")),
    )
    name = tables.WrappingColumn(
        "name", verbose_name=_("Name"),
        link="horizon:admin:share_group_snapshots:detail")
    description = tables.Column(
        "description",
        verbose_name=_("Description"),
        truncate=40)
    created_at = tables.Column(
        "created_at",
        verbose_name=_("Created at"),
        filters=(
            filters.parse_isotime,
        ))
    status = tables.Column(
        "status",
        filters=(title,),
        verbose_name=_("Status"),
        status=True,
        status_choices=STATUS_CHOICES,
        display_choices=STATUS_DISPLAY_CHOICES)
    source = ShareGroupSnapshotShareGroupNameColumn(
        "share_group",
        verbose_name=_("Source"),
        link="horizon:admin:share_groups:detail")

    def get_object_display(self, obj):
        return obj.name

    class Meta(object):
        name = "share_group_snapshots"
        verbose_name = _("Share Group Snapshots")
        status_columns = ["status"]
        row_class = UpdateShareGroupSnapshotRow
        table_actions = (
            tables.NameFilterAction,
            DeleteShareGroupSnapshot,
        )
        row_actions = (
            ResetShareGroupSnapshotStatus,
            DeleteShareGroupSnapshot,
        )



# Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from django.urls import reverse
from unittest import mock

from manila_ui.api import manila as api_manila
from manila_ui.tests.dashboards.project import test_data
from manila_ui.tests import helpers as test

from openstack_dashboard.api import neutron

SHARE_INDEX_URL = reverse('horizon:admin:shares:index')


@ddt.ddt
class ReplicasTests(test.BaseAdminViewTests):

    def setUp(self):
        super(self.__class__, self).setUp()
        self.share = test_data.share
        self.share_replica = test_data.share_replica
        self.share_replica2 = test_data.share_replica2
        self.mock_object(
            api_manila, "share_get", mock.Mock(return_value=self.share))
        self.mock_object(
            neutron, "is_service_enabled", mock.Mock(return_value=[True]))

    def test_detail(self):
        url = reverse(
            "horizon:admin:shares:replica_detail",
            args=[self.share_replica.id])
        export_locations = test_data.export_locations
        self.mock_object(
            api_manila, "share_replica_get",
            mock.Mock(return_value=self.share_replica))
        self.mock_object(
            api_manila, "share_instance_export_location_list",
            mock.Mock(return_value=export_locations))

        res = self.client.get(url)

        self.assertEqual(200, res.status_code)
        self.assertTemplateUsed(res, "admin/shares/replicas/detail.html")
        self.assertNoMessages()
        self.assertContains(res, "<h3>Share Replica Overview", 1, 200)
        self.assertContains(res, ">%s</a>" % self.share.id, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % self.share_replica.id, 1, 200)
        self.assertContains(
            res, "<dd>%s</dd>" % self.share.availability_zone, 1, 200)
        for el in export_locations:
            self.assertContains(res, "value=\"%s\"" % el.path, 1, 200)
            self.assertContains(
                res, "<div><b>Preferred:</b> %s</div>" % el.preferred, 1, 200)
            self.assertContains(
                res, "<div><b>Is admin only:</b> %s</div>" % el.is_admin_only,
                1, 200)
        self.assertNoMessages()
        api_manila.share_replica_get.assert_called_once_with(
            mock.ANY, self.share_replica.id)
        api_manila.share_instance_export_location_list.assert_called_once_with(
            mock.ANY, self.share_replica.id)

    def test_detail_not_found(self):
        url = reverse("horizon:admin:shares:replica_detail",
                      args=[self.share_replica.id])
        self.mock_object(
            api_manila,
            "share_replica_get",
            mock.Mock(
                side_effect=Exception("Fake replica NotFound exception")))
        self.mock_object(api_manila, "share_instance_export_location_list")

        res = self.client.get(url)

        self.assertEqual(302, res.status_code)
        self.assertRedirectsNoFollow(res, SHARE_INDEX_URL)
        self.assertTemplateNotUsed(res, "admin/shares/replicas/detail.html")
        api_manila.share_replica_get.assert_called_once_with(
            mock.ANY, self.share_replica.id)
        self.assertFalse(api_manila.share_instance_export_location_list.called)

    def test_list(self):
        self.mock_object(
            api_manila,
            "share_replica_list",
            mock.Mock(return_value=[self.share_replica]))
        url = reverse(
            "horizon:admin:shares:manage_replicas", args=[self.share.id])

        res = self.client.get(url)

        self.assertEqual(200, res.status_code)
        self.assertTemplateUsed(
            res, "admin/shares/replicas/manage_replicas.html")
        api_manila.share_get.assert_called_with(mock.ANY, self.share.id)
        api_manila.share_replica_list.assert_called_with(
            mock.ANY, self.share.id)

    def test_list_exception(self):
        self.mock_object(
            api_manila,
            "share_replica_list",
            mock.Mock(side_effect=Exception("Fake exception")))
        url = reverse(
            "horizon:admin:shares:manage_replicas", args=[self.share.id])

        res = self.client.get(url)

        self.assertEqual(302, res.status_code)
        self.assertRedirectsNoFollow(res, SHARE_INDEX_URL)
        self.assertTemplateNotUsed(
            res, "admin/shares/replicas/manage_replicas.html")
        api_manila.share_replica_list.assert_called_with(
            mock.ANY, self.share.id)

    @ddt.data(
        ([test_data.share_replica], test_data.share_replica.id, None),
        ([test_data.share_replica], test_data.share_replica.id, 'dr'),
        ([test_data.share_replica], test_data.share_replica.id, 'readable'),
        ([test_data.share_replica], test_data.share_replica.id, 'writable'),
        ([test_data.share_replica, test_data.share_replica2],
         test_data.share_replica.id, 'dr'),
        ([test_data.share_replica, test_data.share_replica2],
         test_data.share_replica.id, 'readable'),
    )
    @ddt.unpack
    def test_delete_not_allowed(self, replica_list, replica_id,
                                replication_type):
        share = test_data.share
        share.replication_type = replication_type
        formData = {"action": "replicas__delete__%s" % replica_id}
        self.mock_object(api_manila, "share_replica_delete")
        self.mock_object(
            api_manila, "share_get", mock.Mock(return_value=share))
        self.mock_object(
            api_manila,
            "share_replica_list",
            mock.Mock(return_value=replica_list))
        url = reverse(
            "horizon:admin:shares:manage_replicas", args=[share.id])

        res = self.client.post(url, formData)

        self.assertEqual(302, res.status_code)
        self.assertRedirectsNoFollow(res, url)
        api_manila.share_replica_list.assert_called_with(mock.ANY, share.id)
        self.assertFalse(api_manila.share_replica_delete.called)

    @ddt.data(
        ([test_data.share_replica, test_data.share_replica2],
         test_data.share_replica2.id, 'dr'),
        ([test_data.share_replica, test_data.share_replica2],
         test_data.share_replica2.id, 'readable'),
        ([test_data.share_replica, test_data.share_replica3],
         test_data.share_replica.id, 'writable'),
        ([test_data.share_replica, test_data.share_replica3],
         test_data.share_replica3.id, 'writable'),
    )
    @ddt.unpack
    def test_delete_allowed(self, replica_list, replica_id, replication_type):
        share = test_data.share
        share.replication_type = replication_type
        formData = {"action": "replicas__delete__%s" % replica_id}
        self.mock_object(api_manila, "share_replica_delete")
        self.mock_object(
            api_manila, "share_get", mock.Mock(return_value=share))
        self.mock_object(
            api_manila,
            "share_replica_list", mock.Mock(return_value=replica_list))
        url = reverse(
            "horizon:admin:shares:manage_replicas", args=[share.id])

        res = self.client.post(url, formData)

        self.assertEqual(302, res.status_code)
        self.assertRedirectsNoFollow(res, url)
        api_manila.share_replica_list.assert_called_with(mock.ANY, share.id)
        api_manila.share_replica_delete.assert_called_with(
            mock.ANY, replica_id)

    def test_resync_replica_get(self):
        url = reverse(
            "horizon:admin:shares:resync_replica",
            args=[self.share_replica.id])
        self.mock_object(api_manila, "share_replica_get")
        self.mock_object(api_manila, "share_replica_resync")

        res = self.client.get(url)

        self.assertEqual(200, res.status_code)
        self.assertTemplateUsed(
            res, "admin/shares/replicas/resync_replica.html")
        self.assertFalse(api_manila.share_replica_get.called)
        self.assertFalse(api_manila.share_replica_resync.called)

    def _resync_post(self):
        url = reverse(
            "horizon:admin:shares:resync_replica",
            args=[self.share_replica.id])
        self.mock_object(
            api_manila,
            "share_replica_get",
            mock.Mock(return_value=self.share_replica))
        formData = {
            "method": "ResyncReplicaForm",
            "replica_id": self.share_replica.id,
        }

        res = self.client.post(url, formData)

        self.assertEqual(302, res.status_code)
        self.assertTemplateNotUsed(
            res, "admin/shares/replicas/resync_replica.html")
        api_manila.share_replica_resync.assert_called_once_with(
            mock.ANY, self.share_replica)
        return res

    def test_resync_post(self):
        self.mock_object(api_manila, "share_replica_resync")

        res = self._resync_post()

        self.assertRedirectsNoFollow(
            res,
            reverse("horizon:admin:shares:manage_replicas",
                    args=[self.share.id]))

    def test_resync_post_error(self):
        self.mock_object(
            api_manila,
            "share_replica_resync",
            mock.Mock(side_effect=Exception("Fake exception")))

        res = self._resync_post()

        self.assertRedirectsNoFollow(
            res, reverse("horizon:admin:shares:index"))

    def test_reset_replica_status_get(self):
        url = reverse(
            "horizon:admin:shares:reset_replica_status",
            args=[self.share_replica.id])
        self.mock_object(api_manila, "share_replica_get")
        self.mock_object(api_manila, "share_replica_reset_status")

        res = self.client.get(url)

        self.assertEqual(200, res.status_code)
        self.assertTemplateUsed(
            res, "admin/shares/replicas/reset_replica_status.html")
        self.assertFalse(api_manila.share_replica_get.called)
        self.assertFalse(api_manila.share_replica_reset_status.called)

    @ddt.data("available", "creating", "deleting", "error")
    def test_reset_replica_status_post(self, status):
        url = reverse(
            "horizon:admin:shares:reset_replica_status",
            args=[self.share_replica.id])
        self.mock_object(
            api_manila, "share_replica_get",
            mock.Mock(return_value=self.share_replica))
        self.mock_object(api_manila, "share_replica_reset_status")
        form_data = {"replica_status": status}

        res = self.client.post(url, form_data)

        self.assertEqual(302, res.status_code)
        self.assertTemplateNotUsed(
            res, "admin/shares/replicas/reset_replica_status.html")
        api_manila.share_replica_reset_status.assert_called_once_with(
            mock.ANY, self.share_replica, status)
        self.assertRedirectsNoFollow(
            res,
            reverse("horizon:admin:shares:manage_replicas",
                    args=[self.share.id]))

    def test_reset_replica_status_error(self):
        status = "error"
        url = reverse(
            "horizon:admin:shares:reset_replica_status",
            args=[self.share_replica.id])
        self.mock_object(
            api_manila, "share_replica_get",
            mock.Mock(return_value=self.share_replica))
        self.mock_object(
            api_manila,
            "share_replica_reset_status",
            mock.Mock(side_effect=Exception("Fake exception")))
        form_data = {"replica_status": status}

        res = self.client.post(url, form_data)

        self.assertEqual(302, res.status_code)
        self.assertTemplateNotUsed(
            res, "admin/shares/replicas/reset_replica_status.html")
        api_manila.share_replica_reset_status.assert_called_once_with(
            mock.ANY, self.share_replica, status)
        self.assertRedirectsNoFollow(
            res, reverse("horizon:admin:shares:index"))

    def test_reset_replica_state_get(self):
        url = reverse(
            "horizon:admin:shares:reset_replica_state",
            args=[self.share_replica.id])
        self.mock_object(api_manila, "share_replica_get")
        self.mock_object(api_manila, "share_replica_reset_state")

        res = self.client.get(url)

        self.assertEqual(200, res.status_code)
        self.assertTemplateUsed(
            res, "admin/shares/replicas/reset_replica_state.html")
        self.assertFalse(api_manila.share_replica_get.called)
        self.assertFalse(api_manila.share_replica_reset_state.called)

    @ddt.data("active", "in_sync", "out_of_sync", "error")
    def test_reset_replica_state_post(self, state):
        url = reverse(
            "horizon:admin:shares:reset_replica_state",
            args=[self.share_replica.id])
        self.mock_object(
            api_manila, "share_replica_get",
            mock.Mock(return_value=self.share_replica))
        self.mock_object(api_manila, "share_replica_reset_state")
        form_data = {"replica_state": state}

        res = self.client.post(url, form_data)

        self.assertEqual(302, res.status_code)
        self.assertTemplateNotUsed(
            res, "admin/shares/replicas/reset_replica_state.html")
        api_manila.share_replica_reset_state.assert_called_once_with(
            mock.ANY, self.share_replica, state)
        self.assertRedirectsNoFollow(
            res,
            reverse("horizon:admin:shares:manage_replicas",
                    args=[self.share.id]))

    def test_reset_replica_state_error(self):
        state = "error"
        url = reverse(
            "horizon:admin:shares:reset_replica_state",
            args=[self.share_replica.id])
        self.mock_object(
            api_manila, "share_replica_get",
            mock.Mock(return_value=self.share_replica))
        self.mock_object(
            api_manila,
            "share_replica_reset_state",
            mock.Mock(side_effect=Exception("Fake exception")))
        form_data = {"replica_state": state}

        res = self.client.post(url, form_data)

        self.assertEqual(302, res.status_code)
        self.assertTemplateNotUsed(
            res, "admin/shares/replicas/reset_replica_state.html")
        api_manila.share_replica_reset_state.assert_called_once_with(
            mock.ANY, self.share_replica, state)
        self.assertRedirectsNoFollow(
            res, reverse("horizon:admin:shares:index"))



# Copyright (c) 2015 Mirantis, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt

from openstack_dashboard.api import base as horizon_api

from manila_ui.api import manila as api
from manila_ui.tests import helpers as base


@ddt.ddt
class ManilaApiTests(base.APITestCase):

    def setUp(self):
        super(self.__class__, self).setUp()
        self.id = "fake_id"
        self.mock_object(horizon_api, "QuotaSet")

    # Share tests

    @ddt.data(
        {},
        None,
        {"name": "fake_share"},
        {"limit": "3"},
        {"host": "fake_share_host"},
        {"sort_key": "id", "sort_dir": "asc"},
    )
    def test_share_list(self, kwargs):
        result = api.share_list(self.request, search_opts=kwargs)

        self.assertEqual(
            self.manilaclient.shares.list.return_value, result)

        expected_kwargs = kwargs or {}
        self.manilaclient.shares.list.assert_called_once_with(
            search_opts=expected_kwargs)

    def test_share_get(self):
        api.share_get(self.request, self.id)

        self.manilaclient.shares.get.assert_called_once_with(self.id)

    @ddt.data((None, True), ("some_fake_sg_id", False))
    @ddt.unpack
    def test_share_create(self, sg_id, is_public):
        kwargs = {
            "share_network": "fake_sn",
            "snapshot_id": "fake_snapshot_id",
            "metadata": {"k1": "v1", "k2": "v2"},
            "share_type": "fake_st",
            "is_public": is_public,
            "availability_zone": "fake_az",
            "share_group_id": sg_id,
        }
        size = 5
        name = "fake_name"
        desc = "fake_description"
        proto = "fake_share_protocol"

        api.share_create(self.request, size, name, desc, proto, **kwargs)

        self.manilaclient.shares.create.assert_called_once_with(
            proto, size, name=name, description=desc, **kwargs)

    @ddt.data(None, "some_fake_sg_id")
    def test_share_delete(self, sg_id):
        s_id = "fake_share_id"

        api.share_delete(self.request, s_id, sg_id)

        self.manilaclient.shares.delete.assert_called_once_with(
            s_id, share_group_id=sg_id)

    @ddt.data(("", "", True), ("share_name", "", True),
              ("share_name", "share_description", True),
              ("", "share_description", True),
              ("", "", False), ("share_name", "", False),
              ("share_name", "share_description", False),
              ("", "share_description", False))
    @ddt.unpack
    def test_share_update(self, name, description, is_public):
        api.share_update(self.request, self.id, name, description, is_public)

        self.manilaclient.shares.update.assert_called_once_with(
            self.id, display_name=name, display_description=description,
            is_public=is_public)

    # Shares ACLs tests

    def test_share_rules_list(self):
        result = api.share_rules_list(self.request, self.id)

        self.assertEqual(
            self.manilaclient.share_access_rules.
            access_list.return_value, result)

        self.manilaclient.share_access_rules.\
            access_list.assert_called_once_with(self.id)

    def test_share_rule_get(self):
        result = api.share_rule_get(self.request, self.id)

        self.assertEqual(
            self.manilaclient.share_access_rules.get.
            return_value, result)

        self.manilaclient.share_access_rules.get.\
            assert_called_once_with(self.id)

    def test_share_rule_set_metadata(self):
        fake_metadata = {
            "aim": "testing",
            "project": "test",
        }

        api.share_rule_set_metadata(
            self.request, self.id, fake_metadata)

        self.manilaclient.share_access_rules.\
            set_metadata.assert_called_once_with(self.id, fake_metadata)

    def test_share_rule_unset_metadata(self):
        fake_key = "test"

        api.share_rule_unset_metadata(
            self.request, self.id, fake_key)

        self.manilaclient.share_access_rules.\
            unset_metadata.assert_called_once_with(self.id, fake_key)

    def test_list_share_export_locations(self):
        api.share_export_location_list(self.request, self.id)

        self.manilaclient.share_export_locations.list.assert_called_once_with(
            self.id)

    def test_list_share_instance_export_locations(self):
        api.share_instance_export_location_list(self.request, self.id)

        client = self.manilaclient
        client.share_instance_export_locations.list.assert_called_once_with(
            self.id)

    @ddt.data(("ip", "10.0.0.13", "rw", {'testkey': "testval"}),
              ("ip", "10.0.0.13", None, None),
              ("ip", "10.0.0.13", "ro", {'key1': "val1"}),
              ("user", "demo", "rw", {'key2': "val2"}),
              ("user", "demo", None, None),
              ("user", "demo", "ro", None),
              ("cephx", "alice", "rw", {'test': "some"}),
              ("cephx", "alice", None, None),
              ("cephx", "alice", "ro", None))
    @ddt.unpack
    def test_share_allow(self, access_type, access_to, access_level, metadata):
        api.share_allow(self.request, self.id, access_type,
                        access_to, access_level, metadata)

        self.manilaclient.shares.allow.assert_called_once_with(
            self.id, access_type, access_to, access_level, metadata)

    def test_share_deny(self):
        fake_rule_id = "fake_rule_id"
        api.share_deny(self.request, self.id, fake_rule_id)

        self.manilaclient.shares.deny.assert_called_once_with(
            self.id, fake_rule_id)

    # Share manage/unmanage tests

    def test_share_manage(self):
        api.share_manage(
            self.request,
            service_host="fake_service_host",
            protocol="fake_protocol",
            export_path="fake_export_path",
            driver_options={"fake_key": "fake_value"},
            share_type="fake_share_type",
            name="fake_name",
            description="fake_description",
            is_public="fake_is_public",
        )

        self.manilaclient.shares.manage.assert_called_once_with(
            service_host="fake_service_host",
            protocol="fake_protocol",
            export_path="fake_export_path",
            driver_options={"fake_key": "fake_value"},
            share_type="fake_share_type",
            name="fake_name",
            description="fake_description",
            is_public="fake_is_public",
        )

    def test_share_unmanage(self):
        api.share_unmanage(self.request, self.id)

        self.manilaclient.shares.unmanage.assert_called_once_with(self.id)

    # Share migration tests

    def test_migration_start(self):
        api.migration_start(self.request, 'fake_share', 'fake_host', False,
                            True, True, True, True, 'fake_net_id',
                            'fake_type_id')

        self.manilaclient.shares.migration_start.assert_called_once_with(
            'fake_share',
            host='fake_host',
            force_host_assisted_migration=False,
            nondisruptive=True,
            writable=True,
            preserve_metadata=True,
            preserve_snapshots=True,
            new_share_network_id='fake_net_id',
            new_share_type_id='fake_type_id'
        )

    def test_migration_complete(self):
        api.migration_complete(self.request, 'fake_share')

        self.manilaclient.shares.migration_complete.assert_called_once_with(
            'fake_share')

    def test_migration_cancel(self):
        api.migration_cancel(self.request, 'fake_share')
        self.manilaclient.shares.migration_cancel.assert_called_once_with(
            'fake_share')

    @ddt.data(
        ('name_updated_1', 'description_updated', True),
        ('name_updated_2', 'description_updated', False),
        ('name_updated_3', None, True),
    )
    @ddt.unpack
    def test_share_type_update(
            self, name, description, is_public):
        api.share_type_update(self.request, self.id, name,
                              description, is_public)

        self.manilaclient.share_types.get.return_value.update.\
            assert_called_once_with(name=name,
                                    description=description,
                                    is_public=is_public)

    def test_migration_get_progress(self):
        api.migration_get_progress(self.request, 'fake_share')

        (self.manilaclient.shares.migration_get_progress.
            assert_called_once_with('fake_share'))

    # Share resize tests
    @ddt.data(("123", "78"), ("2", "5"),
              ("75", "21"), ("0", "2"),
              ("18", "62"), ("-1", "3"))
    @ddt.unpack
    def test_share_resize(self, new_size, orig_size):
        api.share_resize(self.request, self.id, new_size, orig_size)
        if orig_size > new_size:
            self.manilaclient.shares.shrink.assert_called_once_with(
                self.id, new_size)
        else:
            self.manilaclient.shares.extend.assert_called_once_with(
                self.id, new_size)

    # Share snapshots tests

    def test_share_revert(self):
        share = 'fake_share'
        snapshot = 'fake_snapshot'

        api.share_revert(self.request, share, snapshot)

        self.manilaclient.shares.revert_to_snapshot.assert_called_once_with(
            share, snapshot)

    def test_share_snapshot_get(self):
        snapshot = 'fake_snapshot'

        api.share_snapshot_get(self.request, snapshot)

        self.manilaclient.share_snapshots.get.assert_called_once_with(snapshot)

    @ddt.data(("", ""), ("share_name", ""),
              ("share_name", "share_description"),
              ("", "share_description"),)
    @ddt.unpack
    def test_share_snapshot_update(self, name, description):
        snapshot = 'fake_snapshot'

        api.share_snapshot_update(self.request, snapshot, name, description)

        self.manilaclient.share_snapshots.update.assert_called_once_with(
            snapshot, display_name=name, display_description=description)

    @ddt.data(
        {},
        {"detailed": True},
        {"detailed": False},
        {"search_opts": {"foo": "bar"}},
        {"sort_key": "id", "sort_dir": "asc"},
    )
    def test_share_snapshot_list(self, kwargs):
        result = api.share_snapshot_list(self.request, **kwargs)

        self.assertEqual(
            self.manilaclient.share_snapshots.list.return_value, result)
        self.manilaclient.share_snapshots.list.assert_called_once_with(
            detailed=kwargs.get("detailed", True),
            search_opts=kwargs.get("search_opts"),
            sort_key=kwargs.get("sort_key"),
            sort_dir=kwargs.get("sort_dir"),
        )

    @ddt.data(True, False)
    def test_snapshot_create(self, force):
        name = 'fake_snapshot_name'
        description = "fake_snapshot_description"

        api.share_snapshot_create(
            self.request, self.id, name, description, force)

        self.manilaclient.share_snapshots.create.assert_called_once_with(
            self.id, name=name, description=description, force=force)

    def test_snapshot_delete(self):
        snapshot_id = 'fake_snapshot_id'

        api.share_snapshot_delete(self.request, snapshot_id)

        self.manilaclient.share_snapshots.delete.assert_called_once_with(
            snapshot_id)

    def test_allow_snapshot(self):
        access_type = "fake_type"
        access_to = "fake_value"

        api.share_snapshot_allow(self.request, self.id, access_type,
                                 access_to)

        client = self.manilaclient
        client.share_snapshots.allow.assert_called_once_with(
            self.id, access_type, access_to)

    def test_deny_snapshot(self):
        api.share_snapshot_deny(self.request, self.id, self.id)

        client = self.manilaclient
        client.share_snapshots.deny.assert_called_once_with(self.id, self.id)

    def test_list_snapshot_rules(self):
        api.share_snapshot_rules_list(self.request, self.id)

        client = self.manilaclient
        client.share_snapshots.access_list.assert_called_once_with(self.id)

    def test_list_snapshot_export_locations(self):
        api.share_snap_export_location_list(self.request, self.id)

        client = self.manilaclient
        client.share_snapshot_export_locations.list.assert_called_once_with(
            snapshot=self.id)

    def test_list_snapshot_instance_export_locations(self):
        api.share_snap_instance_export_location_list(self.request, self.id)

        client = self.manilaclient
        client.share_snapshot_export_locations.list.assert_called_once_with(
            snapshot_instance=self.id)

    # Share network tests

    @ddt.data(
        {},
        {"detailed": True},
        {"detailed": False},
        {"detailed": False, "search_opts": {"foo": "bar"}},
    )
    def test_share_network_list(self, kwargs):
        api.share_network_list(self.request, **kwargs)

        self.manilaclient.share_networks.list.assert_called_once_with(
            detailed=kwargs.get("detailed", False),
            search_opts=kwargs.get("search_opts", None))

    @ddt.data(
        {},
        {"name": "foo_name"},
        {"description": "foo_desc"},
        {"neutron_net_id": "foo_neutron_net_id"},
        {"neutron_subnet_id": "foo_neutron_subnet_id"},
        {"name": "foo_name", "description": "foo_desc",
         "neutron_net_id": "foo_neutron_net_id",
         "neutron_subnet_id": "foo_neutron_subnet_id"},
    )
    def test_share_network_create(self, kwargs):
        expected_kwargs = {
            "name": None,
            "description": None,
            "neutron_net_id": None,
            "neutron_subnet_id": None,
            "availability_zone": None
        }
        expected_kwargs.update(**kwargs)

        api.share_network_create(self.request, **kwargs)

        mock_sn_create = self.manilaclient.share_networks.create
        mock_sn_create.assert_called_once_with(**expected_kwargs)

    def test_share_network_get(self):
        share_net_id = 'fake_share_net_id'

        api.share_network_get(self.request, share_net_id)

        self.manilaclient.share_networks.get.assert_called_once_with(
            share_net_id)

    @ddt.data(("", ""), ("share_net_name", ""),
              ("share_net_name", "share_net_description"),
              ("", "share_net_description"))
    @ddt.unpack
    def test_share_network_update(self, name, description):
        share_net_id = "fake_share_network"

        api.share_network_update(self.request, share_net_id, name, description)

        self.manilaclient.share_networks.update.assert_called_once_with(
            share_net_id, name=name, description=description)

    def test_share_network_delete(self):
        share_network_id = "fake_share_network"

        api.share_network_delete(self.request, share_network_id)

        self.manilaclient.share_networks.delete.assert_called_once_with(
            share_network_id)

    @ddt.data(
        {"share_network_id": "some_network_id"},
        {"share_network_id": "some_network_id",
         "neutron_net_id": "some_neutron_net_id",
         "neutron_subnet_id": "some_neutron_subnet_id",
         "availability_zone": "some_availability_zone"},
    )
    def test_share_network_subnet_create(self, kwargs):
        expected_kwargs = {
            "share_network_id": None,
            "neutron_net_id": None,
            "neutron_subnet_id": None,
            "availability_zone": None
        }

        expected_kwargs.update(**kwargs)
        api.share_network_subnet_create(self.request, **kwargs)
        self.manilaclient.share_network_subnets.create.assert_called_once_with(
            **expected_kwargs)

    # Share server tests

    @ddt.data(
        {},
        {"name": "fake_share_server"},
        {"limit": "10"},
        {"host": "fake_share_server_host"},
        {"sort_key": "id", "sort_dir": "asc"},
    )
    def test_share_server_list(self, kwargs):
        result = api.share_server_list(self.request, search_opts=kwargs)

        self.assertEqual(
            self.manilaclient.share_servers.list.return_value, result)

        self.manilaclient.share_servers.list.assert_called_once_with(
            search_opts=kwargs)

    def test_share_server_get(self):
        share_serv_id = 'fake_share_server'
        api.share_server_get(self.request, share_serv_id)

        self.manilaclient.share_servers.get.assert_called_once_with(
            share_serv_id)

    def test_share_server_delete(self):
        share_serv_id = "fake_share_server"

        api.share_server_delete(self.request, share_serv_id)

        self.manilaclient.share_servers.delete.assert_called_once_with(
            share_serv_id)

    # Security service tests

    @ddt.data(
        {},
        {"search_opts": {"foo": "bar"}},
    )
    def test_security_service_list(self, kwargs):
        api.security_service_list(self.request, **kwargs)

        self.manilaclient.security_services.list.assert_called_once_with(
            detailed=True,
            search_opts=kwargs.get("search_opts", None))

    def test_security_service_get(self):
        sec_service_id = 'fake_sec_service_id'
        api.security_service_get(self.request, sec_service_id)

        self.manilaclient.security_services.get.assert_called_once_with(
            sec_service_id)

    @ddt.data(
        ("ldap", {}),
        ("kerberos", {}),
        ("ldap",
            {"dns_ip": "8.8.8.8", "ou": "fakeOU",
             "name": "my_fake_ldap_security_service",
             "description": "LDAP security service"}),
        ("kerberos",
            {"server": "10.254.0.3",
             "user": "demo", "password": "s3cr37",
             "name": "my_fake_kerberos_security_service",
             "description": "Kerberos security service"})
    )
    @ddt.unpack
    def test_security_service_create(self, ss_type, kwargs):
        expected_kwargs = {
            "dns_ip": None,
            "ou": None,
            "server": None,
            "domain": None,
            "password": None,
            "user": None,
            "name": None,
            "description": None
        }
        expected_kwargs.update(**kwargs)

        api.security_service_create(self.request, ss_type, **kwargs)

        mock_sec_service_create = self.manilaclient.security_services.create
        mock_sec_service_create.assert_called_once_with(
            ss_type, **expected_kwargs)

    @ddt.data(
        {"dns_ip": "8.8.4.4", "ou": "testOU",
         "server": "10.254.0.3", "domain": "None",
         "password": "wr67p6", "user": "demo",
         "name": "my_fake_ldap_security_service_2",
         "description": "LDAP security service 2"},
        {"dns_ip": "None", "ou": "None",
         "server": "10.254.0.10", "domain": "None",
         "password": "None", "user": "demo",
         "name": "my_fake_kerberos_security_service_2",
         "description": "Kerberos security service 2"}
    )
    def test_security_service_update(self, kwargs):
        sec_service_id = "fake_sec_service_id"

        api.security_service_update(self.request, sec_service_id, **kwargs)

        mock_sec_service_update = self.manilaclient.security_services.update
        mock_sec_service_update.assert_called_once_with(
            sec_service_id, **kwargs)

    def test_security_service_delete(self):
        sec_service_id = "fake_sec_service_id"

        api.security_service_delete(self.request, sec_service_id)

        self.manilaclient.security_services.delete.assert_called_once_with(
            sec_service_id)

    # Share network security service tests

    def test_share_network_security_service_add(self):
        share_network_id = "fake_share_net_id"
        sec_service_id = "fake_sec_service_id"

        api.share_network_security_service_add(
            self.request, share_network_id, sec_service_id)

        mock_sn_sec_service_add = (
            self.manilaclient.share_networks.add_security_service)

        mock_sn_sec_service_add.assert_called_once_with(
            share_network_id, sec_service_id)

    def test_share_network_security_service_remove(self):
        share_network_id = "fake_share_net_id"
        sec_service_id = "fake_sec_service_id"

        api.share_network_security_service_remove(
            self.request, share_network_id, sec_service_id)

        mock_sn_sec_service_rm = (
            self.manilaclient.share_networks.remove_security_service)

        mock_sn_sec_service_rm.assert_called_once_with(
            share_network_id, sec_service_id)

    def test_share_network_security_service_list(self):
        share_network_id = "fake_share_net_id"

        api.share_network_security_service_list(
            self.request, share_network_id)

        search_opts = {"share_network_id": share_network_id}

        self.manilaclient.security_services.list.assert_called_once_with(
            search_opts=search_opts)

    # Share metadata tests

    def test_share_set_metadata(self):
        fake_metadata = {
            "aim": "testing",
            "project": "my_abc",
            "deadline": "01/01/2020"
        }

        api.share_set_metadata(
            self.request, self.id, fake_metadata)

        self.manilaclient.shares.set_metadata.assert_called_once_with(
            self.id, fake_metadata)

    def test_share_delete_metadata(self):
        fake_keys = ["aim", "project", "deadline"]

        api.share_delete_metadata(
            self.request, self.id, fake_keys)

        self.manilaclient.shares.delete_metadata.assert_called_once_with(
            self.id, fake_keys)

    # Default quota tests

    def test_default_quota_get(self):
        project_id = 'fake_project_id'

        api.tenant_quota_get(self.request, project_id)

        self.manilaclient.quotas.get.assert_called_once_with(project_id)

    @ddt.data(
        ({'share_gigabytes': 333}, {'gigabytes': 333}),
        ({'share_snapshot_gigabytes': 444}, {'snapshot_gigabytes': 444}),
        ({'share_snapshots': 14}, {'snapshots': 14}),
        ({'snapshots': 14}, {'snapshots': 14}),
        ({'gigabytes': 14}, {'gigabytes': 14}),
        ({'snapshot_gigabytes': 314}, {'snapshot_gigabytes': 314}),
        ({'shares': 24}, {'shares': 24}),
        ({'share_networks': 14}, {'share_networks': 14}),
    )
    @ddt.unpack
    def test_tenant_quota_update(self, provided_kwargs, expected_kwargs):
        tenant_id = 'fake_tenant_id'

        api.tenant_quota_update(self.request, tenant_id, **provided_kwargs)

        self.manilaclient.quotas.update.assert_called_once_with(
            tenant_id, **expected_kwargs)
        self.manilaclient.quota_classes.update.assert_not_called()

    @ddt.data(
        ({'share_gigabytes': 333}, {'gigabytes': 333}),
        ({'share_snapshot_gigabytes': 444}, {'snapshot_gigabytes': 444}),
        ({'share_snapshots': 14}, {'snapshots': 14}),
        ({'snapshots': 14}, {'snapshots': 14}),
        ({'gigabytes': 14}, {'gigabytes': 14}),
        ({'snapshot_gigabytes': 314}, {'snapshot_gigabytes': 314}),
        ({'shares': 24}, {'shares': 24}),
        ({'share_networks': 14}, {'share_networks': 14}),
    )
    @ddt.unpack
    def test_default_quota_update(self, provided_kwargs, expected_kwargs):
        api.default_quota_update(self.request, **provided_kwargs)

        self.manilaclient.quota_classes.update.assert_called_once_with(
            api.DEFAULT_QUOTA_NAME, **expected_kwargs)

    def test_tenant_quota_get(self):
        tenant_id = 'fake_tenant_id'
        result = api.tenant_quota_get(self.request, tenant_id)

        self.assertIsNotNone(result)
        self.manilaclient.quotas.get.assert_called_once_with(tenant_id)

    @ddt.data({
        'shares': 24, 'gigabytes': 333, 'snapshots': 14,
        'snapshot_gigabytes': 444, 'share_networks': 14,
        "share_groups": 30, "share_group_snapshots": 5,
    })
    @ddt.unpack
    def test_ui_data_map(self, **kwargs):
        expected_result = {
            'shares': 24, 'share_gigabytes': 333, 'share_snapshots': 14,
            'share_snapshot_gigabytes': 444, 'share_networks': 14,
            "share_groups": 30, "share_group_snapshots": 5,
        }

        converted_result_for_ui = {}
        for field in api.MANILA_QUOTA_FIELDS:
            converted_result_for_ui[field] = (
                kwargs[api.MANILA_QUOTA_FIELDS_DATA_MAP[field]])

        self.assertEqual(expected_result, converted_result_for_ui)

    # Share type tests

    def test_share_type_list(self):
        api.share_type_list(self.request)

        self.manilaclient.share_types.list.assert_called_once_with()

    def test_share_type_get(self):
        share_type_id = "fake_share_type_id"

        api.share_type_get(self.request, share_type_id)

        self.manilaclient.share_types.get.assert_called_once_with(
            share_type_id)

    @ddt.data(True, False)
    def test_share_type_create_with_default_values(self, dhss):
        name = 'fake_share_type_name'

        api.share_type_create(self.request, name, dhss)

        self.manilaclient.share_types.create.assert_called_once_with(
            name=name,
            spec_driver_handles_share_servers=dhss,
            description=None,
            is_public=True)

    @ddt.data(
        (True, "fake_description", True),
        (False, "fake_description", True),
        (True, "fake_description", False),
        (False, "fake_description", False),
    )
    @ddt.unpack
    def test_share_type_create_with_custom_values(
            self, dhss, description, is_public):
        name = 'fake_share_type_name'

        api.share_type_create(
            self.request, name, dhss, description,
            is_public)

        self.manilaclient.share_types.create.assert_called_once_with(
            name=name,
            description=description,
            spec_driver_handles_share_servers=dhss,
            is_public=is_public)

    def test_share_type_delete(self):
        share_type_id = "fake_share_type_id"

        api.share_type_delete(self.request, share_type_id)

        self.manilaclient.share_types.delete.assert_called_once_with(
            share_type_id)

    def test_share_type_get_extra_specs(self):
        share_type_id = "fake_share_type_id"

        api.share_type_get_extra_specs(self.request, share_type_id)

        share_types_get = self.manilaclient.share_types.get
        share_types_get.assert_called_once_with(share_type_id)
        share_types_get.return_value.get_keys.assert_called_once()

    def test_share_type_set_extra_specs(self):
        data = {"foo": "bar"}

        api.share_type_set_extra_specs(self.request, self.id, data)

        share_types_get = self.manilaclient.share_types.get
        share_types_get.assert_called_once_with(self.id)
        share_types_get.return_value.set_keys.assert_called_once_with(data)

    def test_share_type_unset_extra_specs(self):
        keys = ["foo", "bar"]

        api.share_type_unset_extra_specs(self.request, self.id, keys)

        share_types_get = self.manilaclient.share_types.get
        share_types_get.assert_called_once_with(self.id)
        share_types_get.return_value.unset_keys.assert_called_once_with(keys)

    def test_share_type_access_list(self):
        share_type_id = "fake_share_type_id"

        api.share_type_access_list(self.request, share_type_id)

        self.manilaclient.share_type_access.list.assert_called_once_with(
            share_type_id)

    def test_share_type_access_add(self):
        share_type_id = "fake_share_type_id"
        project_id = "fake_project_id"

        api.share_type_access_add(self.request, share_type_id, project_id)

        mock_share_type_access_add = (
            self.manilaclient.share_type_access.add_project_access)

        mock_share_type_access_add.assert_called_once_with(
            share_type_id, project_id)

    def test_share_type_access_remove(self):
        share_type_id = "fake_share_type_id"
        project_id = "fake_project_id"

        api.share_type_access_remove(self.request, share_type_id, project_id)

        mock_share_type_access_rm = (
            self.manilaclient.share_type_access.remove_project_access)

        mock_share_type_access_rm.assert_called_once_with(
            share_type_id, project_id)

    # Share replica tests

    def test_share_replica_list(self):
        api.share_replica_list(self.request)

        self.manilaclient.share_replicas.list.assert_called_once_with(None)

    def test_share_replica_list_with_filter_by_share(self):
        api.share_replica_list(self.request, share="FOO")

        self.manilaclient.share_replicas.list.assert_called_once_with("FOO")

    def test_share_replica_create(self):
        share = "FOO_share"
        availability_zone = "BAR_availability_zone"

        api.share_replica_create(
            self.request, share, availability_zone)

        self.manilaclient.share_replicas.create.assert_called_once_with(
            share,
            availability_zone=availability_zone
        )

    def test_share_replica_get(self):
        api.share_replica_get(self.request, "fake")

        self.manilaclient.share_replicas.get.assert_called_once_with("fake")

    def test_share_replica_delete(self):
        api.share_replica_delete(self.request, "fake")

        self.manilaclient.share_replicas.delete.assert_called_once_with("fake")

    def test_share_replica_promote(self):
        api.share_replica_promote(self.request, "fake")

        self.manilaclient.share_replicas.promote.assert_called_once_with(
            "fake")

    def test_share_replica_resync(self):
        api.share_replica_resync(self.request, "fake")

        self.manilaclient.share_replicas.resync.assert_called_once_with("fake")

    def test_share_replica_reset_status(self):
        replica = "fake_replica"
        status = "fake_status"

        api.share_replica_reset_status(self.request, replica, status)

        self.manilaclient.share_replicas.reset_state.assert_called_once_with(
            replica, status)

    def test_share_replica_reset_state(self):
        replica = "fake_replica"
        state = "fake_state"

        api.share_replica_reset_state(self.request, replica, state)

        mock_reset_state = self.manilaclient.share_replicas.reset_replica_state
        mock_reset_state.assert_called_once_with(replica, state)

    # Share instance tests

    def test_share_instance_list(self):
        api.share_instance_list(self.request)

        self.manilaclient.share_instances.list.assert_called_once_with()

    def test_share_instance_get(self):
        api.share_instance_get(self.request, self.id)

        self.manilaclient.share_instances.get.assert_called_once_with(self.id)

    # Availability zone and pool tests

    def test_availability_zone_list(self):
        api.availability_zone_list(self.request)

        self.manilaclient.availability_zones.list.assert_called_once_with()

    @ddt.data({}, {"detailed": True}, {"detailed": False})
    def test_pool_list(self, kwargs):
        api.pool_list(self.request, **kwargs)

        self.manilaclient.pools.list.assert_called_once_with(
            detailed=kwargs.get("detailed", False))

    # Share groups tests

    def test_share_group_create(self):
        name = "fake_sg_name"
        kwargs = {
            "description": "fake_desc",
            "share_group_type": "fake_sg_type",
            "share_types": ["fake", "list", "of", "fake", "share", "types"],
            "share_network": "fake_sn",
            "source_share_group_snapshot": "fake_source_share_group_snapshot",
            "availability_zone": "fake_az",
        }

        result = api.share_group_create(self.request, name, **kwargs)

        self.assertEqual(
            self.manilaclient.share_groups.create.return_value, result)
        self.manilaclient.share_groups.create.assert_called_once_with(
            name=name, **kwargs)

    def test_share_group_get(self):
        sg = "fake_share_group"

        result = api.share_group_get(self.request, sg)

        self.assertEqual(
            self.manilaclient.share_groups.get.return_value, result)
        self.manilaclient.share_groups.get.assert_called_once_with(sg)

    def test_share_group_update(self):
        sg = "fake_share_group"
        name = "fake_name"
        desc = "fake_desc"

        result = api.share_group_update(self.request, sg, name, desc)

        self.assertEqual(
            self.manilaclient.share_groups.update.return_value, result)
        self.manilaclient.share_groups.update.assert_called_once_with(
            sg, name=name, description=desc)

    @ddt.data({}, {"force": True}, {"force": False})
    def test_share_group_delete(self, kwargs):
        sg = 'fake_share_group'

        api.share_group_delete(self.request, sg, **kwargs)

        self.manilaclient.share_groups.delete.assert_called_once_with(
            sg, force=kwargs.get("force", False))

    def test_share_group_reset_state(self):
        sg = 'fake_share_group'
        state = 'fake_state'

        result = api.share_group_reset_state(self.request, sg, state)

        self.assertIsNotNone(result)
        self.assertEqual(
            self.manilaclient.share_groups.reset_state.return_value,
            result)
        self.manilaclient.share_groups.reset_state.assert_called_once_with(
            sg, state)

    @ddt.data(
        {},
        {"detailed": True},
        {"detailed": False},
        {"search_opts": {"foo": "bar"}},
        {"sort_key": "id", "sort_dir": "asc"},
    )
    def test_share_group_list(self, kwargs):
        result = api.share_group_list(self.request, **kwargs)

        self.assertEqual(
            self.manilaclient.share_groups.list.return_value, result)
        self.manilaclient.share_groups.list.assert_called_once_with(
            detailed=kwargs.get("detailed", True),
            search_opts=kwargs.get("search_opts"),
            sort_key=kwargs.get("sort_key"),
            sort_dir=kwargs.get("sort_dir"),
        )

    # Share Group Snapshots tests

    def test_share_group_snapshot_create(self):
        sg = 'fake_share_group'
        name = 'fake_name'
        desc = 'fake_description'

        result = api.share_group_snapshot_create(self.request, sg, name, desc)

        self.assertIsNotNone(result)
        self.assertEqual(
            self.manilaclient.share_group_snapshots.create.return_value,
            result)
        self.manilaclient.share_group_snapshots.create.assert_called_once_with(
            share_group=sg, name=name, description=desc)

    def test_share_group_snapshot_get(self):
        sgs = 'fake_share_group_snapshot'

        result = api.share_group_snapshot_get(self.request, sgs)

        self.assertIsNotNone(result)
        self.assertEqual(
            self.manilaclient.share_group_snapshots.get.return_value, result)
        self.manilaclient.share_group_snapshots.get.assert_called_once_with(
            sgs)

    def test_share_group_snapshot_update(self):
        sgs = 'fake_share_group_snapshot'
        name = 'fake_name'
        desc = 'fake_description'

        result = api.share_group_snapshot_update(self.request, sgs, name, desc)

        self.assertIsNotNone(result)
        self.assertEqual(
            self.manilaclient.share_group_snapshots.update.return_value,
            result)
        self.manilaclient.share_group_snapshots.update.assert_called_once_with(
            sgs, name=name, description=desc)

    @ddt.data(True, False)
    def test_share_group_snapshot_delete(self, force):
        sgs = 'fake_share_group_snapshot'

        result = api.share_group_snapshot_delete(self.request, sgs, force)

        self.assertIsNotNone(result)
        self.assertEqual(
            self.manilaclient.share_group_snapshots.delete.return_value,
            result)
        self.manilaclient.share_group_snapshots.delete.assert_called_once_with(
            sgs, force=force)

    def test_share_group_snapshot_reset_state(self):
        sgs = 'fake_share_group_snapshot'
        state = 'fake_state'

        result = api.share_group_snapshot_reset_state(self.request, sgs, state)

        rs_method = self.manilaclient.share_group_snapshots.reset_state
        self.assertIsNotNone(result)
        self.assertEqual(rs_method.return_value, result)
        rs_method.assert_called_once_with(sgs, state)

    @ddt.data(
        {},
        {'detailed': False},
        {'detailed': True, 'search_opts': 'foo',
         'sort_key': 'k', 'sort_dir': 'v'},
    )
    def test_share_group_snapshot_list(self, kwargs):
        result = api.share_group_snapshot_list(self.request, **kwargs)

        self.assertIsNotNone(result)
        self.assertEqual(
            self.manilaclient.share_group_snapshots.list.return_value,
            result)
        self.manilaclient.share_group_snapshots.list.assert_called_once_with(
            detailed=kwargs.get('detailed', True),
            search_opts=kwargs.get('search_opts'),
            sort_key=kwargs.get('sort_key'),
            sort_dir=kwargs.get('sort_dir'))

    # Share Group Types tests

    @ddt.data(
        {'is_public': True},
        {'is_public': False, 'group_specs': {'foo': 'bar'}},
        {'group_specs': {}},
    )
    def test_share_group_type_create(self, kwargs):
        name = 'fake_sgt_name'
        sts = ['fake', 'list', 'of', 'share', 'types']

        result = api.share_group_type_create(self.request, name, sts, **kwargs)

        self.assertIsNotNone(result)
        self.assertEqual(
            self.manilaclient.share_group_types.create.return_value,
            result)
        self.manilaclient.share_group_types.create.assert_called_once_with(
            name=name, share_types=sts,
            is_public=kwargs.get('is_public', False),
            group_specs=kwargs.get('group_specs'))

    def test_share_group_type_get(self):
        sgt = "fake_sgt"

        result = api.share_group_type_get(self.request, sgt)

        self.assertIsNotNone(result)
        self.assertEqual(
            self.manilaclient.share_group_types.get.return_value, result)
        self.manilaclient.share_group_types.get.assert_called_once_with(sgt)

    @ddt.data(True, False)
    def test_share_group_type_list(self, show_all):
        result = api.share_group_type_list(self.request, show_all)

        self.assertIsNotNone(result)
        self.assertEqual(
            self.manilaclient.share_group_types.list.return_value, result)
        self.manilaclient.share_group_types.list.assert_called_once_with(
            show_all=show_all)

    def test_share_group_type_delete(self):
        sgt = 'fake_share_group_type'

        result = api.share_group_type_delete(self.request, sgt)

        self.assertIsNotNone(result)
        self.assertEqual(
            self.manilaclient.share_group_types.delete.return_value, result)
        self.manilaclient.share_group_types.delete.assert_called_once_with(sgt)

    def test_share_group_type_access_list(self):
        sgt = 'fake_share_group_type'

        result = api.share_group_type_access_list(self.request, sgt)

        self.assertIsNotNone(result)
        self.assertEqual(
            self.manilaclient.share_group_type_access.list.return_value,
            result)
        self.manilaclient.share_group_type_access.list.assert_called_once_with(
            sgt)

    def test_share_group_type_access_add(self):
        sgt = 'fake_share_group_type'
        project = 'fake_project'

        result = api.share_group_type_access_add(self.request, sgt, project)

        sgt_access = self.manilaclient.share_group_type_access
        self.assertIsNotNone(result)
        self.assertEqual(
            sgt_access.add_project_access.return_value, result)
        sgt_access.add_project_access.assert_called_once_with(sgt, project)

    def test_share_group_type_access_remove(self):
        sgt = 'fake_share_group_type'
        project = 'fake_project'

        result = api.share_group_type_access_remove(self.request, sgt, project)

        sgt_access = self.manilaclient.share_group_type_access
        self.assertIsNotNone(result)
        self.assertEqual(
            sgt_access.remove_project_access.return_value, result)
        sgt_access.remove_project_access.assert_called_once_with(sgt, project)

    def test_share_group_type_set_specs(self):
        sgt = 'fake_share_group_type'
        group_specs = 'fake_specs'

        result = api.share_group_type_set_specs(self.request, sgt, group_specs)

        get_method = self.manilaclient.share_group_types.get
        self.assertIsNotNone(result)
        self.assertEqual(get_method.return_value.set_keys.return_value, result)
        get_method.assert_called_once_with(sgt)
        get_method.return_value.set_keys.assert_called_once_with(group_specs)

    def test_share_group_type_unset_specs(self):
        sgt = 'fake_share_group_type'
        keys = ['fake', 'list', 'of', 'keys', 'for', 'deletion']

        result = api.share_group_type_unset_specs(self.request, sgt, keys)

        get_method = self.manilaclient.share_group_types.get
        self.assertIsNotNone(result)
        self.assertEqual(
            get_method.return_value.unset_keys.return_value, result)
        get_method.assert_called_once_with(sgt)
        get_method.return_value.unset_keys.assert_called_once_with(keys)

    def test_share_group_type_get_specs(self):
        sgt = 'fake_share_group_type'

        result = api.share_group_type_get_specs(self.request, sgt)

        get_method = self.manilaclient.share_group_types.get
        self.assertIsNotNone(result)
        self.assertEqual(
            get_method.return_value.get_keys.return_value, result)
        get_method.assert_called_once_with(sgt)
        get_method.return_value.get_keys.assert_called_once_with()

    @ddt.data(None, "some_fake_message_id")
    def test_share_messages_get(self, message_id):
        result = api.messages_get(self.request, message_id)

        self.assertIsNotNone(result)
        self.assertEqual(
            self.manilaclient.messages.get.return_value,
            result)
        self.manilaclient.messages.get.assert_called_once_with(message_id)

    @ddt.data(
        {},
        {'search_opts': 'foo', 'sort_key': 'k', 'sort_dir': 'v'},
    )
    def test_messages_list(self, kwargs):
        result = api.messages_list(self.request, **kwargs)

        self.assertIsNotNone(result)
        self.assertEqual(
            self.manilaclient.messages.list.return_value,
            result)
        self.manilaclient.messages.list.assert_called_once_with(
            search_opts=kwargs.get('search_opts'),
            sort_key=kwargs.get('sort_key'),
            sort_dir=kwargs.get('sort_dir'))

    @ddt.data(None, "some_fake_message_id")
    def test_messages_delete(self, message_id):
        result = api.messages_delete(self.request, message_id)

        self.assertIsNotNone(result)
        self.assertEqual(
            self.manilaclient.messages.delete.return_value,
            result)
        self.manilaclient.messages.delete.assert_called_once_with(message_id)



# Copyright 2017 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Admin views for managing share networks.
"""
import re

from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import tables
from horizon.utils import memoized
from manila_ui.api import manila
from manila_ui.dashboards.admin.share_networks import tables as sn_tables
from manila_ui.dashboards.admin.share_networks import tabs as sn_tabs
from manila_ui.dashboards.admin import utils
from manila_ui.dashboards.project.share_networks import views as p_views


class ShareNetworksView(tables.MultiTableView):
    table_classes = (
        sn_tables.ShareNetworksTable,
    )
    template_name = "admin/share_networks/index.html"
    page_title = _("Share Networks")

    @memoized.memoized_method
    def get_share_networks_data(self):
        try:
            share_networks = manila.share_network_list(
                self.request, detailed=True, search_opts={"all_tenants": True})
        except Exception:
            share_networks = []
            exceptions.handle(
                self.request, _("Unable to retrieve share networks"))
        utils.set_project_name_to_objects(self.request, share_networks)
        share_networks = self.get_filters(share_networks)
        return share_networks

    def get_filters(self, share_networks):
        table = self._tables['share_networks']
        filters = self.get_server_filter_info(table.request, table)
        filter_string = filters['value']
        filter_field = filters['field']
        if filter_string and filter_field:
            filtered_data = []
            for share_network in share_networks:
                if filter_field == 'name':
                    if share_network.name == filter_string:
                        filtered_data.append(share_network)

                if filter_field == 'description':
                    re_string = re.compile(filter_string)
                    if (share_network.description and
                            re.search(re_string,
                                      share_network.description)):
                        filtered_data.append(share_network)
            return filtered_data
        else:
            return share_networks


class ShareNetworkDetailView(p_views.Detail):
    tab_group_class = sn_tabs.ShareNetworkDetailTabs
    template_name = "admin/share_networks/detail.html"
    redirect_url = reverse_lazy("horizon:admin:share_networks:index")



# Copyright 2017 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import re_path

from manila_ui.dashboards.project.share_snapshots import views


urlpatterns = [
    re_path(
        r'^$',
        views.ShareSnapshotsView.as_view(),
        name='index'),
    re_path(
        r'^(?P<share_id>[^/]+)/share_snapshot_create/$',
        views.CreateShareSnapshotView.as_view(),
        name='share_snapshot_create'),
    re_path(
        r'^(?P<snapshot_id>[^/]+)/share_snapshot_edit/$',
        views.UpdateShareSnapshotView.as_view(),
        name='share_snapshot_edit'),
    re_path(
        r'^(?P<snapshot_id>[^/]+)$',
        views.ShareSnapshotDetailView.as_view(),
        name='share_snapshot_detail'),
    re_path(
        r'^(?P<snapshot_id>[^/]+)/share_snapshot_rules/$',
        views.ManageShareSnapshotRulesView.as_view(),
        name='share_snapshot_manage_rules'),
    re_path(
        r'^(?P<snapshot_id>[^/]+)/share_snapshot_rule_add/$',
        views.AddShareSnapshotRuleView.as_view(),
        name='share_snapshot_rule_add'),
]



# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import tables
from horizon import tabs
from horizon.utils import memoized

from manila_ui.api import manila
from manila_ui.dashboards.project.shares import forms as share_form
from manila_ui.dashboards.project.shares import tables as shares_tables
from manila_ui.dashboards.project.shares import tabs as shares_tabs
from manila_ui.dashboards import utils as ui_utils


class ShareTableMixIn(object):
    def _get_shares(self, search_opts=None):
        try:
            return manila.share_list(self.request, search_opts=search_opts)
        except Exception:
            exceptions.handle(self.request,
                              _('Unable to retrieve share list.'))
            return []

    def _set_id_if_nameless(self, shares):
        for share in shares:
            # It is possible to create a share with no name
            if not share.name:
                share.name = share.id


class SharesView(tables.MultiTableView, ShareTableMixIn):
    table_classes = (
        shares_tables.SharesTable,
    )
    template_name = "project/shares/index.html"
    page_title = _("Shares")

    @memoized.memoized_method
    def get_shares_data(self):
        share_nets_names = {}
        share_nets = manila.share_network_list(self.request)
        for share_net in share_nets:
            share_nets_names[share_net.id] = share_net.name
        try:
            shares = manila.share_list(self.request)
            for share in shares:
                share.share_network = (
                    share_nets_names.get(share.share_network_id) or
                    share.share_network_id)
                share.metadata = ui_utils.metadata_to_str(share.metadata)

            snapshots = manila.share_snapshot_list(self.request, detailed=True)
            share_ids_with_snapshots = []
            for snapshot in snapshots:
                share_ids_with_snapshots.append(snapshot.to_dict()['share_id'])
            for share in shares:
                if share.to_dict()['id'] in share_ids_with_snapshots:
                    setattr(share, 'has_snapshot', True)
                else:
                    setattr(share, 'has_snapshot', False)
        except Exception:
            exceptions.handle(
                self.request, _('Unable to retrieve share list.'))
            return []
        # Gather our tenants to correlate against IDs
        return shares


class DetailView(tabs.TabView):
    tab_group_class = shares_tabs.ShareDetailTabs
    template_name = 'project/shares/detail.html'

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        share = self.get_data()
        share_display_name = share.name or share.id
        context["share"] = share
        context["share_display_name"] = share_display_name
        context["page_title"] = _("Share Details: "
                                  "%(share_display_name)s") % {
            'share_display_name': share_display_name}
        return context

    @memoized.memoized_method
    def get_data(self):
        try:
            share_id = self.kwargs['share_id']
            share = manila.share_get(self.request, share_id)
            share.rules = manila.share_rules_list(self.request, share_id)
            share.export_locations = manila.share_export_location_list(
                self.request, share_id)
            export_locations = [
                exp['path'] for exp in share.export_locations]
            share.el_size = ui_utils.calculate_longest_str_size(
                export_locations)
        except Exception:
            redirect = reverse('horizon:project:shares:index')
            exceptions.handle(self.request,
                              _('Unable to retrieve share details.'),
                              redirect=redirect)
        return share

    def get_tabs(self, request, *args, **kwargs):
        share = self.get_data()
        return self.tab_group_class(request, share=share, **kwargs)


class CreateView(forms.ModalFormView):
    form_class = share_form.CreateForm
    form_id = "create_share"
    template_name = 'project/shares/create.html'
    modal_header = _("Create Share")
    modal_id = "create_share_modal"
    submit_label = _("Create")
    submit_url = reverse_lazy("horizon:project:shares:create")
    success_url = reverse_lazy("horizon:project:shares:index")
    page_title = _('Create a Share')

    def get_context_data(self, **kwargs):
        context = super(CreateView, self).get_context_data(**kwargs)
        try:
            context['usages'] = manila.tenant_absolute_limits(self.request)
        except Exception:
            exceptions.handle(self.request)
        return context


class UpdateView(forms.ModalFormView):
    form_class = share_form.UpdateForm
    form_id = "update_share"
    template_name = 'project/shares/update.html'
    modal_header = _("Edit Share")
    modal_id = "update_share_modal"
    submit_label = _("Edit")
    submit_url = "horizon:project:shares:update"
    success_url = reverse_lazy("horizon:project:shares:index")
    page_title = _('Edit Share')

    def get_object(self):
        if not hasattr(self, "_object"):
            vol_id = self.kwargs['share_id']
            try:
                self._object = manila.share_get(self.request, vol_id)
            except Exception:
                msg = _('Unable to retrieve share.')
                url = reverse('horizon:project:shares:index')
                exceptions.handle(self.request, msg, redirect=url)
        return self._object

    def get_context_data(self, **kwargs):
        context = super(UpdateView, self).get_context_data(**kwargs)
        return context

    def get_initial(self):
        self.submit_url = reverse(self.submit_url, kwargs=self.kwargs)
        share = self.get_object()
        return {'share_id': self.kwargs["share_id"],
                'name': share.name,
                'description': share.description,
                'is_public': share.is_public}


class UpdateMetadataView(forms.ModalFormView):
    form_class = share_form.UpdateMetadataForm
    form_id = "update_share_metadata"
    template_name = 'project/shares/update_metadata.html'
    modal_header = _("Edit Share Metadata")
    modal_id = "update_share_metadata_modal"
    submit_label = _("Save Changes")
    submit_url = "horizon:project:shares:update_metadata"
    success_url = reverse_lazy("horizon:project:shares:index")
    page_title = _('Edit Share Metadata')

    def get_object(self):
        if not hasattr(self, "_object"):
            sh_id = self.kwargs['share_id']
            try:
                self._object = manila.share_get(self.request, sh_id)
            except Exception:
                msg = _('Unable to retrieve share.')
                url = reverse('horizon:project:shares:index')
                exceptions.handle(self.request, msg, redirect=url)
        return self._object

    def get_context_data(self, **kwargs):
        context = super(UpdateMetadataView, self).get_context_data(**kwargs)
        args = (self.get_object().id,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        share = self.get_object()
        return {'share_id': self.kwargs["share_id"],
                'metadata': share.metadata}


class AddRuleView(forms.ModalFormView):
    form_class = share_form.AddRule
    form_id = "rule_add"
    template_name = 'project/shares/rule_add.html'
    modal_header = _("Add Rule")
    modal_id = "rule_add_modal"
    submit_label = _("Add")
    submit_url = "horizon:project:shares:rule_add"
    success_url = reverse_lazy("horizon:project:shares:index")
    page_title = _('Add Rule')

    def get_object(self):
        if not hasattr(self, "_object"):
            vol_id = self.kwargs['share_id']
            try:
                self._object = manila.share_get(self.request, vol_id)
            except Exception:
                msg = _('Unable to retrieve share.')
                url = reverse('horizon:project:shares:index')
                exceptions.handle(self.request, msg, redirect=url)
        return self._object

    def get_context_data(self, **kwargs):
        context = super(AddRuleView, self).get_context_data(**kwargs)
        args = (self.get_object().id,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        share = self.get_object()
        return {'share_id': self.kwargs["share_id"],
                'name': share.name,
                'description': share.description}

    def get_success_url(self):
        return reverse("horizon:project:shares:manage_rules",
                       args=[self.kwargs['share_id']])


class UpdateRuleMetadataView(forms.ModalFormView):
    form_class = share_form.UpdateRuleMetadataForm
    form_id = ""
    template_name = 'project/shares/update_rule_metadata.html'
    modal_header = _("Edit Rule Metadata")
    modal_id = "update_rule_metadata_modal"
    submit_label = _("Save Changes")
    submit_url = "horizon:project:shares:update_rule_metadata"
    success_url = reverse_lazy("horizon:project:shares:index")
    page_title = _('Edit Rule Metadata')

    def get_object(self):
        if not hasattr(self, "_object"):
            rule_id = self.kwargs['rule_id']
            try:
                self._object = manila.share_rule_get(self.request, rule_id)
            except Exception:
                msg = _('Unable to retrieve share access rule.')
                url = reverse('horizon:project:shares:manage_rules',
                              args=[self.initial['share_id']])
                exceptions.handle(self.request, msg, redirect=url)
        return self._object

    def get_context_data(self, **kwargs):
        context = super(UpdateRuleMetadataView, self).\
            get_context_data(**kwargs)
        args = (self.get_object().id,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        rule = self.get_object()
        rule_id = self.kwargs["rule_id"]
        share_id = self.get_share_id(rule_id)
        return {'rule_id': rule_id,
                'metadata': rule.metadata,
                'share_id': share_id}

    def get_success_url(self):
        rule_id = self.kwargs["rule_id"]
        share_id = self.get_share_id(rule_id)
        return reverse("horizon:project:shares:manage_rules", args=[share_id])

    # To redirect to Rules Table page, after updating the rule
    # metadata. Loop is to get the share_id neccessary to display
    # Rules Table page.
    def get_share_id(self, rule_id):
        shares = manila.share_list(self.request)
        for share in shares:
            rules = manila.share_rules_list(self.request, share.id)
            for rule in rules:
                if rule.id == rule_id:
                    return share.id


class ManageRulesView(tables.DataTableView):
    table_class = shares_tables.RulesTable
    template_name = 'project/shares/manage_rules.html'

    def get_context_data(self, **kwargs):
        context = super(ManageRulesView, self).get_context_data(**kwargs)
        share = manila.share_get(self.request, self.kwargs['share_id'])
        context['share_display_name'] = share.name or share.id
        context["page_title"] = _("Share Rules: "
                                  "%(share_display_name)s") % {
            'share_display_name': context['share_display_name']}
        return context

    @memoized.memoized_method
    def get_data(self):
        try:
            share_id = self.kwargs['share_id']
            rules = manila.share_rules_list(self.request, share_id)
        except Exception:
            redirect = reverse('horizon:project:shares:index')
            exceptions.handle(self.request,
                              _('Unable to retrieve share rules.'),
                              redirect=redirect)
            return []

        for rule in rules:
            rule.metadata = ui_utils.metadata_to_str(rule.metadata)

        return rules


class ResizeView(forms.ModalFormView):
    form_class = share_form.ResizeForm
    form_id = "resize_share"
    template_name = 'project/shares/resize.html'
    modal_header = _("Resize Share")
    modal_id = "resize_share_modal"
    submit_label = _("Resize")
    submit_url = "horizon:project:shares:resize"
    success_url = reverse_lazy("horizon:project:shares:index")
    page_title = _('Resize Share')

    @memoized.memoized_method
    def get_object(self):
        try:
            return manila.share_get(self.request, self.kwargs['share_id'])
        except Exception:
            exceptions.handle(self.request, _('Unable to retrieve share.'))

    def get_context_data(self, **kwargs):
        context = super(ResizeView, self).get_context_data(**kwargs)
        args = (self.get_object().id,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        try:
            context['usages'] = manila.tenant_absolute_limits(self.request)
            context['usages']['totalShareGigabytesUsed'] -= int(
                self.get_object().size)
        except Exception:
            exceptions.handle(self.request)

        return context

    def get_initial(self):
        share = self.get_object()
        if not share or isinstance(share, Exception):
            raise exceptions.NotFound()
        return {
            'share_id': self.kwargs["share_id"],
            'name': share.name or share.id,
            'orig_size': share.size,
            'new_size': int(share.size),
        }


class RevertView(forms.ModalFormView):
    form_class = share_form.RevertForm
    form_id = "revert_share"
    template_name = 'project/shares/revert.html'
    modal_header = _("Revert Share to a Snapshot")
    modal_id = "revert_share_modal"
    submit_label = _("Revert share to a snapshot")
    submit_url = "horizon:project:shares:revert"
    success_url = reverse_lazy("horizon:project:shares:index")
    page_title = _('Revert Share to a Snapshot')

    @memoized.memoized_method
    def get_object(self):
        try:
            return manila.share_get(self.request, self.kwargs['share_id'])
        except Exception:
            exceptions.handle(self.request, _('Unable to retrieve share.'))

    def get_context_data(self, **kwargs):
        context = super(self.__class__, self).get_context_data(**kwargs)
        args = (self.get_object().id,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        share = self.get_object()
        if not share or isinstance(share, Exception):
            raise exceptions.NotFound()
        return {
            'share_id': self.kwargs["share_id"],
            'name': share.name or share.id,
        }



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.utils.translation import gettext_lazy as _
from horizon import tables


class CreateShareNetworkSubnet(tables.LinkAction):
    name = "subnet"
    verbose_name = _("Create Subnet")
    url = "horizon:project:share_networks:share_network_subnet_create"
    classes = ("ajax-modal", "btn-create")
    policy_rules = (("share", "share_network_subnet:create"),)

    def allowed(self, request, obj_id):
        return True



# Copyright (c) 2020 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.utils.translation import gettext_lazy as _
from horizon import tables

from manila_ui.dashboards.project.user_messages import tables as project_tables


class DeleteUserMessage(project_tables.DeleteUserMessage):
    pass


class UserMessageIDColumn(project_tables.UserMessageIDColumn):
    pass


class UserMessagesAdminTable(project_tables.UserMessagesTable):
    project_name = tables.Column(
        "project_name",
        verbose_name=_("Project"),
    )

    class Meta(object):
        name = "user_messages"
        verbose_name = _("User Messages")
        table_actions = (
            tables.NameFilterAction,
            DeleteUserMessage,
        )
        row_actions = (
            DeleteUserMessage,
        )
        columns = (
            'project_name', 'message_id', 'message_level',
            'resource_type',
            'resource_id', 'user_message', 'created_at',
        )



# Copyright (c) 2014 NetApp, Inc.
# Copyright (c) 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from horizon import exceptions as horizon_exceptions
from neutronclient.client import exceptions
from openstack_dashboard.api import keystone as api_keystone
from openstack_dashboard.api import neutron as api_neutron
from oslo_utils import timeutils
from unittest import mock

from manila_ui.api import manila as api_manila
from manila_ui.dashboards.admin import share_networks
from manila_ui.dashboards.admin import utils
from manila_ui.tests.dashboards.project import test_data
from manila_ui.tests import helpers as test
from manila_ui.tests.test_data import keystone_data

INDEX_URL = reverse('horizon:admin:share_networks:index')


class ShareNetworksTableMock(mock.Mock):
    def __init__(self, *args, **kwargs):
        mock.Mock.__init__(self)
        self.request = {}
        self.share_networks = {}


class ShareNetwork:
    def __init__(self, id, name, description):
        self.id = id
        self.name = name
        self.description = description


class ShareNetworksTests(test.BaseAdminViewTests):

    def setUp(self):
        super(self.__class__, self).setUp()
        self.mock_object(
            api_keystone, "tenant_list",
            mock.Mock(return_value=(keystone_data.projects, None)))
        # Reset taken list of projects to avoid test interference
        utils.PROJECTS = {}

    class FakeAZ(object):
        def __init__(self, name, id):
            self.name = name
            self.id = id
            self.created_at = timeutils.utcnow()

    def test_detail_view(self):
        share_net = test_data.active_share_network
        share_network_subnets = share_net.share_network_subnets
        sec_service = test_data.sec_service
        self.mock_object(
            api_manila, "share_server_list", mock.Mock(return_value=[]))
        self.mock_object(
            api_manila, "share_network_get", mock.Mock(return_value=share_net))
        self.mock_object(
            api_manila, "share_network_security_service_list",
            mock.Mock(return_value=[sec_service]))
        network = self.networks.first()
        subnet = self.subnets.first()
        self.mock_object(
            api_neutron, "network_get", mock.Mock(return_value=network))
        self.mock_object(
            api_neutron, "subnet_get", mock.Mock(return_value=subnet))
        self.mock_object(
            api_manila, "availability_zone_list",
            mock.Mock(return_value=[self.FakeAZ('fake_az', 'fake_az')])
        )
        url = reverse('horizon:admin:share_networks:share_network_detail',
                      args=[share_net.id])

        res = self.client.get(url)

        self.assertContains(res, "<h1>Share Network Details: %s</h1>"
                                 % share_net.name,
                            1, 200)
        self.assertContains(res, "<dd>%s</dd>" % share_net.name, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % share_net.id, 1, 200)
        for sub in share_network_subnets:
            self.assertContains(res, "<a href=\"/admin/networks"
                                "/%s/detail\">%s</a>" % (
                                    sub['neutron_net_id'],
                                    network.name), 1, 200)
        self.assertContains(res, "<a href=\"/admin/security_services"
                                 "/%s\">%s</a>" % (sec_service.id,
                                                   sec_service.name), 1, 200)
        network_get_calls = [mock.call(mock.ANY, sub['neutron_net_id']
                                       ) for sub in share_network_subnets]
        subnet_get_calls = [mock.call(mock.ANY, sub['neutron_subnet_id']
                                      ) for sub in share_network_subnets]

        api_neutron.network_get.assert_has_calls(network_get_calls,
                                                 any_order=True)
        api_neutron.subnet_get.assert_has_calls(subnet_get_calls,
                                                any_order=True)
        self.assertNoMessages()
        api_manila.share_network_security_service_list.assert_called_once_with(
            mock.ANY, share_net.id)
        api_manila.share_server_list.assert_called_once_with(
            mock.ANY, search_opts={'share_network_id': share_net.id})
        api_manila.share_network_get.assert_called_once_with(
            mock.ANY, share_net.id)

    def test_detail_view_network_not_found(self):
        share_net = test_data.active_share_network
        sec_service = test_data.sec_service
        share_network_subnets = share_net.share_network_subnets
        url = reverse('horizon:admin:share_networks:share_network_detail',
                      args=[share_net.id])
        self.mock_object(
            api_manila, "share_server_list", mock.Mock(return_value=[]))
        self.mock_object(
            api_manila, "share_network_get", mock.Mock(return_value=share_net))
        self.mock_object(
            api_manila, "share_network_security_service_list",
            mock.Mock(return_value=[sec_service]))
        self.mock_object(
            api_neutron, "network_get", mock.Mock(
                side_effect=exceptions.NeutronClientException('fake', 500)))
        self.mock_object(
            api_neutron, "subnet_get", mock.Mock(
                side_effect=exceptions.NeutronClientException('fake', 500)))
        self.mock_object(
            api_manila, "availability_zone_list",
            mock.Mock(return_value=[])
        )
        res = self.client.get(url)

        self.assertContains(res, "<h1>Share Network Details: %s</h1>"
                                 % share_net.name,
                            1, 200)
        self.assertContains(res, "<dd>%s</dd>" % share_net.name, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % share_net.id, 1, 200)
        for sub in share_network_subnets:
            self.assertNotContains(res, "<dd>%s</dd>" % sub['neutron_net_id'])
            self.assertNotContains(res,
                                   "<dd>%s</dd>" % sub['neutron_subnet_id'])
        network_get_calls = [mock.call(mock.ANY, sub['neutron_net_id']
                                       ) for sub in share_network_subnets]
        subnet_get_calls = [mock.call(mock.ANY, sub['neutron_subnet_id']
                                      ) for sub in share_network_subnets]

        api_neutron.network_get.assert_has_calls(network_get_calls,
                                                 any_order=True)
        api_neutron.subnet_get.assert_has_calls(subnet_get_calls,
                                                any_order=True)
        self.assertContains(res, "<a href=\"/admin/security_services"
                                 "/%s\">%s</a>" % (sec_service.id,
                                                   sec_service.name), 1, 200)
        self.assertNoMessages()
        api_manila.share_network_security_service_list.assert_called_once_with(
            mock.ANY, share_net.id)
        api_manila.share_server_list.assert_called_once_with(
            mock.ANY, search_opts={'share_network_id': share_net.id})
        api_manila.share_network_get.assert_called_once_with(
            mock.ANY, share_net.id)

    def test_detail_view_with_exception(self):
        url = reverse('horizon:admin:share_networks:share_network_detail',
                      args=[test_data.active_share_network.id])
        self.mock_object(
            api_manila, "share_network_get",
            mock.Mock(side_effect=horizon_exceptions.NotFound(404)))

        res = self.client.get(url)

        self.assertRedirectsNoFollow(res, INDEX_URL)
        api_manila.share_network_get.assert_called_once_with(
            mock.ANY, test_data.active_share_network.id)

    def test_delete_share_network(self):
        share_network = test_data.inactive_share_network
        formData = {'action': 'share_networks__delete__%s' % share_network.id}
        self.mock_object(api_manila, "share_network_delete")
        self.mock_object(
            api_manila, "share_network_list",
            mock.Mock(return_value=[
                test_data.active_share_network,
                test_data.inactive_share_network]))
        res = self.client.post(INDEX_URL, formData)
        self.assertRedirectsNoFollow(res, INDEX_URL)
        api_keystone.tenant_list.assert_called_once_with(mock.ANY)
        api_manila.share_network_delete.assert_called_once_with(
            mock.ANY, share_network.id)
        api_manila.share_network_list.assert_called_once_with(
            mock.ANY, detailed=True, search_opts={'all_tenants': True})

    def test_get_filters(self):
        share_net = [
            ShareNetwork('share_net_1', 'share_net_name_1', 'desc1'),
            ShareNetwork('share_net_2', 'share_net_name_2', 'desc2'),
            ShareNetwork('share_net_3', 'share_net_name_3', 'desc3'),
        ]
        share_networks_view = share_networks.views.ShareNetworksView()
        share_networks_view._tables = {
            'share_networks': ShareNetworksTableMock()
        }

        mock_filters = {
            'action': '<ShareNetworksFilterAction: filter>',
            'value_param': 'share_networks__filter__q',
            'value': '',
            'field_param': 'share_networks__filter__q_field',
            'field': None,
            'changed': False
        }
        self.mock_object(share_networks_view, 'get_server_filter_info',
                         mock.Mock(return_value=mock_filters))

        # Test with empty filter string and no filter field selected
        filtered_data = share_networks_view.get_filters(share_net)
        self.assertEqual(len(filtered_data), len(share_net))
        self.assertEqual(filtered_data, share_net)

        mock_filters_name = {
            'action': '<ShareNetworksFilterAction: filter>',
            'value_param': 'share_networks__filter__q',
            'value': 'share_net_name_1',
            'field_param': 'share_networks__filter__q_field',
            'field': 'name',
            'changed': False
        }
        self.mock_object(share_networks_view, 'get_server_filter_info',
                         mock.Mock(return_value=mock_filters_name))

        # Test with specific filter string ("share_net_name_1") on "name" field
        filtered_data = share_networks_view.get_filters(share_net)
        self.assertEqual(len(filtered_data), 1)
        self.assertEqual(filtered_data[0].name, 'share_net_name_1')

        mock_filters_description = {
            'action': '<ShareNetworksFilterAction: filter>',
            'value_param': 'share_networks__filter__q',
            'value': 'desc1',
            'field_param': 'share_networks__filter__q_field',
            'field': 'description',
            'changed': False
        }

        self.mock_object(share_networks_view, 'get_server_filter_info',
                         mock.Mock(return_value=mock_filters_description))

        # Test with specific filter string ("desc1") on "description" field
        filtered_data = share_networks_view.get_filters(share_net)
        self.assertEqual(len(filtered_data), 1)
        self.assertEqual(filtered_data[0].description, 'desc1')
